using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Drawing.Imaging;
using System.Net;


namespace SpectralNET {
	/// <summary>
	/// Enumration of different types of eigenvalue/vector pairs
	/// </summary>
	public enum EigenType {
		/// <summary>
		/// The adjacency eigentype (0 means i,j unconnected, 1 means connected).
		/// </summary>
		Adjacency, 

		/// <summary>
		/// The laplacian eigentype (degree along the diagonal, -1 for connected i,j).
		/// </summary>
		Laplacian, 

		/// <summary>
		/// The normalized laplacian eigentype.  1 along the diagonal and -1/root(di*dj) for connected i,j).
		/// </summary>
		NormalizedLaplacian}

	/// <summary>
	/// Enumeration of different display types.  I made this an enum in case this list grows.
	/// </summary>
	public enum displayType {
		/// <summary>
		/// We're currently displaying the F-R graph.
		/// </summary>
		Normal, 

		/// <summary>
		/// We're currently displaying the Laplacian LLE graph.
		/// </summary>
		LaplacianLLE}

	/// <summary>
	/// The Page object that represents this web application.
	/// </summary>
	public class DataAnalyzer : System.Web.UI.Page
	{
		/// <summary>
		/// Wrapper for accessing session state.
		/// </summary>
		public class State 
		{
			/// <summary>
			/// The ChemGraph object associated with this user's session
			/// </summary>
			public static ChemGraph Graph
			{
				get{ return (ChemGraph)HttpContext.Current.Session["g"]; }
				set{ HttpContext.Current.Session["g"] = value; }
			}

			/// <summary>
			/// The user's currently selected vertex
			/// </summary>
			public static Vertex SelectedVertex
			{
				get { return (Vertex)HttpContext.Current.Session["cv"]; }
				set { HttpContext.Current.Session["cv"] = value; }
			}
		}

		// Declare WebForm items here!
		// EXAMPLE: protected System.Web.UI.WebControls.DataList storyList;
		protected System.Web.UI.WebControls.Panel importFile;
		protected System.Web.UI.WebControls.Panel analysisDisplay;
		protected System.Web.UI.WebControls.Panel parseError;
		protected System.Web.UI.HtmlControls.HtmlInputFile inputFileUpload;
		protected System.Web.UI.HtmlControls.HtmlInputFile inputNodeEdgesUpload;
		protected System.Web.UI.WebControls.ListBox verticesLB;
		protected System.Web.UI.WebControls.ListBox edgesLB;
		protected System.Web.UI.WebControls.ImageButton graphDisplay;
		protected System.Web.UI.WebControls.ImageButton laplacianDisplay;
		protected System.Web.UI.WebControls.ImageButton graphDegreeDistribution;
		protected System.Web.UI.WebControls.ImageButton graphClusterCoefficient;
		protected System.Web.UI.WebControls.ImageButton graphPathLength;
		protected System.Web.UI.WebControls.Panel pnlVertex;
		protected System.Web.UI.WebControls.Panel pnlShowComponent;
		protected System.Web.UI.WebControls.Panel pnlVertexComponentInfo;
		protected System.Web.UI.WebControls.Panel pnlComponent;
		protected System.Web.UI.WebControls.Label vName;
		protected System.Web.UI.WebControls.Label vCC;
		protected System.Web.UI.WebControls.Label vDegree;
		protected System.Web.UI.WebControls.Label connInfo;
		protected System.Web.UI.WebControls.Label lblAdjEigens;
		protected System.Web.UI.WebControls.Label lblLapEigens;
		protected System.Web.UI.WebControls.Label lblNormLapEigens;
		protected System.Web.UI.WebControls.RadioButton chemassay;
		protected System.Web.UI.WebControls.RadioButton chemchem;
		protected System.Web.UI.WebControls.DropDownList numSimilarities;
		protected System.Web.UI.WebControls.Label componentName;
		protected System.Web.UI.WebControls.Label vDist;
		protected System.Web.UI.WebControls.Label lblAdjEigensHeader;
		protected System.Web.UI.WebControls.Label lblLapEigensHeader;
		protected System.Web.UI.WebControls.Label lblNormLapEigensHeader;
		protected System.Web.UI.WebControls.Panel pnlEigenvalues;
		protected System.Web.UI.WebControls.Label vAdjE0;
		protected System.Web.UI.WebControls.Label vAdjE1;
		protected System.Web.UI.WebControls.Label vAdjE2;
		protected System.Web.UI.WebControls.Label vAdjE3;
		protected System.Web.UI.WebControls.Label vLapE0;
		protected System.Web.UI.WebControls.Label vLapE1;
		protected System.Web.UI.WebControls.Label vLapE2;
		protected System.Web.UI.WebControls.Label vLapE3;
		protected System.Web.UI.WebControls.Label vNormLapE0;
		protected System.Web.UI.WebControls.Label vNormLapE1;
		protected System.Web.UI.WebControls.Label vNormLapE2;
		protected System.Web.UI.WebControls.Label vNormLapE3;
		protected System.Web.UI.WebControls.ImageButton adjacencyEigenvalues;
		protected System.Web.UI.WebControls.ImageButton adjacencyEigenvectorZero;
		protected System.Web.UI.WebControls.ImageButton adjacencyEigenvectorOne;
		protected System.Web.UI.WebControls.ImageButton adjacencyEigenvectorTwo;
		protected System.Web.UI.WebControls.ImageButton adjacencyEigenvectorThree;
		protected System.Web.UI.WebControls.RadioButton adjE0Vec;
		protected System.Web.UI.WebControls.RadioButton adjE0Degree;
		protected System.Web.UI.WebControls.RadioButton adjE1Vec;
		protected System.Web.UI.WebControls.RadioButton adjE1Degree;
		protected System.Web.UI.WebControls.RadioButton adjE2Vec;
		protected System.Web.UI.WebControls.RadioButton adjE2Degree;
		protected System.Web.UI.WebControls.RadioButton adjE3Vec;
		protected System.Web.UI.WebControls.RadioButton adjE3Degree;
		protected System.Web.UI.WebControls.ImageButton laplacianEigenvalues;
		protected System.Web.UI.WebControls.ImageButton laplacianEigenvectorZero;
		protected System.Web.UI.WebControls.ImageButton laplacianEigenvectorOne;
		protected System.Web.UI.WebControls.ImageButton laplacianEigenvectorTwo;
		protected System.Web.UI.WebControls.ImageButton laplacianEigenvectorThree;
		protected System.Web.UI.WebControls.RadioButton lapE0Vec;
		protected System.Web.UI.WebControls.RadioButton lapE0Degree;
		protected System.Web.UI.WebControls.RadioButton lapE1Vec;
		protected System.Web.UI.WebControls.RadioButton lapE1Degree;
		protected System.Web.UI.WebControls.RadioButton lapE2Vec;
		protected System.Web.UI.WebControls.RadioButton lapE2Degree;
		protected System.Web.UI.WebControls.RadioButton lapE3Vec;
		protected System.Web.UI.WebControls.RadioButton lapE3Degree;
		protected System.Web.UI.WebControls.ImageButton normLaplacianEigenvalues;
		protected System.Web.UI.WebControls.ImageButton normLaplacianEigenvectorZero;
		protected System.Web.UI.WebControls.ImageButton normLaplacianEigenvectorOne;
		protected System.Web.UI.WebControls.ImageButton normLaplacianEigenvectorTwo;
		protected System.Web.UI.WebControls.ImageButton normLaplacianEigenvectorThree;
		protected System.Web.UI.WebControls.RadioButton normLapE0Vec;
		protected System.Web.UI.WebControls.RadioButton normLapE0Degree;
		protected System.Web.UI.WebControls.RadioButton normLapE1Vec;
		protected System.Web.UI.WebControls.RadioButton normLapE1Degree;
		protected System.Web.UI.WebControls.RadioButton normLapE2Vec;
		protected System.Web.UI.WebControls.RadioButton normLapE2Degree;
		protected System.Web.UI.WebControls.RadioButton normLapE3Vec;
		protected System.Web.UI.WebControls.RadioButton normLapE3Degree;
		protected System.Web.UI.WebControls.RadioButton inputFile;
		protected System.Web.UI.WebControls.RadioButton inputNodeEdges;
		protected System.Web.UI.WebControls.RadioButton inputRandomER;
		protected System.Web.UI.WebControls.RadioButton inputRandomScaleFree;
		protected System.Web.UI.WebControls.RadioButton inputRandomHierarchical;
		protected System.Web.UI.WebControls.RadioButton inputRandomSmallWorld;
		protected System.Web.UI.WebControls.RadioButton inputRandomScaleFreeRewiring;
		protected System.Web.UI.WebControls.Label lblEigenInfo;
		protected System.Web.UI.WebControls.Label lblError;
		protected System.Web.UI.WebControls.Button submit;
		protected System.Web.UI.WebControls.TextBox erNumNodes;
		protected System.Web.UI.WebControls.TextBox erProbability;
		protected System.Web.UI.WebControls.TextBox sfNumNodes;
		protected System.Web.UI.WebControls.TextBox sfM;
		protected System.Web.UI.WebControls.TextBox sfrwT;
		protected System.Web.UI.WebControls.TextBox sfrwNewEdges;
		protected System.Web.UI.WebControls.TextBox sfrwRewire;
		protected System.Web.UI.WebControls.TextBox sfrwM;
		protected System.Web.UI.WebControls.TextBox hierCS;
		protected System.Web.UI.WebControls.TextBox hierN;
		protected System.Web.UI.WebControls.ImageButton adjSpec;
		protected System.Web.UI.WebControls.ImageButton lapSpec;
		protected System.Web.UI.WebControls.ImageButton normLapSpec;
		protected System.Web.UI.WebControls.CheckBox doFR;
		protected System.Web.UI.WebControls.TextBox swN;
		protected System.Web.UI.WebControls.TextBox swK;
		protected System.Web.UI.WebControls.TextBox swP;
		protected System.Web.UI.WebControls.ImageButton adjIVP;
		protected System.Web.UI.WebControls.ImageButton lapIVP;
		protected System.Web.UI.WebControls.ImageButton normLapIVP;
		protected System.Web.UI.WebControls.Label lblNormLapIVP;
		protected System.Web.UI.WebControls.Label lblLapIVP;
		protected System.Web.UI.WebControls.Label lblAdjIVP;
		protected System.Web.UI.WebControls.Button submitFile;
		protected System.Web.UI.WebControls.LinkButton gNormal;
		protected System.Web.UI.WebControls.LinkButton gLaplacian;
		protected System.Web.UI.WebControls.Panel normalPanel;
		protected System.Web.UI.WebControls.Panel laplacianPanel;
		protected System.Web.UI.WebControls.Button selectVertex;
		protected System.Web.UI.WebControls.Button collapsePanel;
		protected System.Web.UI.WebControls.Button btnCollapse;
		protected System.Web.UI.WebControls.Panel pnlCollapse;
		protected System.Web.UI.WebControls.ListBox collapseLB;
		protected System.Web.UI.WebControls.TextBox newNodeName;
		protected System.Web.UI.WebControls.Button btnCancelCollapse;
		protected System.Web.UI.WebControls.TextBox chemRegex;
		protected System.Web.UI.WebControls.Button btnTextOutput;
		protected System.Web.UI.WebControls.Panel pnlTextOutput;
		protected System.Web.UI.WebControls.Label lblOps;
		protected System.Web.UI.WebControls.Button btnCollapsePanel;
		protected System.Web.UI.WebControls.Panel pnlOps;
		protected System.Web.UI.WebControls.CheckBox outNode;
		protected System.Web.UI.WebControls.CheckBox outNodeDegree;
		protected System.Web.UI.WebControls.CheckBox outNodeCK;
		protected System.Web.UI.WebControls.CheckBox outNodeAvgDist;
		protected System.Web.UI.WebControls.CheckBox outNodeMaxDist;
		protected System.Web.UI.WebControls.CheckBox outNodeAdjEig;
		protected System.Web.UI.WebControls.CheckBox outNodeLapEig;
		protected System.Web.UI.WebControls.CheckBox outNodeNormLapEig;
		protected System.Web.UI.WebControls.Label lblLabel;
		protected System.Web.UI.WebControls.Label lblIDNumber;
		protected System.Web.UI.WebControls.CheckBox outAdjEigenvalueInfo;
		protected System.Web.UI.WebControls.CheckBox outAdjIVP;
		protected System.Web.UI.WebControls.CheckBox outAdjSpec;
		protected System.Web.UI.WebControls.CheckBox outAdjEigenvector;
		protected System.Web.UI.WebControls.CheckBox outLapEigenvalueInfo;
		protected System.Web.UI.WebControls.CheckBox outLapIVP;
		protected System.Web.UI.WebControls.CheckBox outLapSpec;
		protected System.Web.UI.WebControls.CheckBox outLapEigenvector;
		protected System.Web.UI.WebControls.CheckBox outNormLapEigenvalueInfo;
		protected System.Web.UI.WebControls.CheckBox outNormLapIVP;
		protected System.Web.UI.WebControls.CheckBox outNormLapSpec;
		protected System.Web.UI.WebControls.CheckBox outNormLapEigenvector;
		protected System.Web.UI.WebControls.CheckBox outAdjMat;
		protected System.Web.UI.WebControls.CheckBox outLapMat;
		protected System.Web.UI.WebControls.CheckBox outNormLapMat;
		protected System.Web.UI.WebControls.Panel pnlExportLinks;
		protected System.Web.UI.WebControls.Button btnDoneExport;
		protected System.Web.UI.WebControls.HyperLink linkNodeInfo;
		protected System.Web.UI.WebControls.HyperLink linkAdjEigenvalueInfo;
		protected System.Web.UI.WebControls.HyperLink linkLapEigenvalueInfo;
		protected System.Web.UI.WebControls.HyperLink linkNormLapEigenvalueInfo;
		protected System.Web.UI.WebControls.HyperLink linkAdjMat;
		protected System.Web.UI.WebControls.HyperLink linkLapMat;
		protected System.Web.UI.WebControls.HyperLink linkNormLapMat;
		protected System.Web.UI.WebControls.CheckBox outNodeLabel;
		protected System.Web.UI.WebControls.CheckBox outNodeID;
		protected System.Web.UI.WebControls.CheckBox outAdjEigenvalue;
		protected System.Web.UI.WebControls.CheckBox outLapEigenvalue;
		protected System.Web.UI.WebControls.CheckBox outNormLapEigenvalue;
		protected System.Web.UI.WebControls.Button btnExport;
		protected System.Web.UI.WebControls.Button btnCancelTextOutput;
		protected System.Web.UI.WebControls.CheckBox outNodeLLE;

		Pen whitePen = new Pen(Color.White, 1);
		Pen grayPen = new Pen(Color.Gray, 1);
		Pen blackPen = new Pen(Color.Black, 1);
		Pen bigBlackPen = new Pen(Color.Black, 2);
		Pen dgPen = new Pen(Color.DarkGray, 1);
		Pen pinkPen = new Pen(Color.Pink, 1);
		Pen orangePen = new Pen(Color.Orange, 2);
		Pen redPen = new Pen(Color.Red, 1);
		Brush whiteBrush = new SolidBrush(Color.White);
		Brush grayBrush = new SolidBrush(Color.Gray);
		Brush redBrush = new SolidBrush(Color.FromArgb(255, 51, 0));
		Brush greenBrush = new SolidBrush(Color.Green);
		Brush blackBrush = new SolidBrush(Color.Black);
		Brush orangeBrush = new SolidBrush(Color.Orange);
		Brush pinkBrush = new SolidBrush(Color.Pink);
		Brush darkGreenBrush = new SolidBrush(Color.DarkGreen);
		Brush siennaBrush = new SolidBrush(Color.FromArgb(0, 51, 153));

		/// <summary>
		/// Displays the graph for the first time -- sets up, parses, etc. the graph
		/// </summary>
		/// <param name="sender">The submit button</param>
		/// <param name="e">Unused</param>
		protected void submit_Click(Object sender, EventArgs e)
		{
			if (inputFile.Checked == true)
			{
				StreamReader sr = new StreamReader(inputFileUpload.PostedFile.InputStream);

				// The array of integers for parsing
				//char[] ints = new char[] {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
				string[] results = null; // to store the results of the split
				char[] delims = new char[] {' '};

				try
				{
					Trace.Write("Parsing beginning...", "");

					// PARSE: sr.ReadLine().ToString()
					string curline = sr.ReadLine().ToString();

					//
					// PHASE 1: Parse out *Vertices #NUM line
					//
					// get rid of whitespace
					while (curline.Trim() == "") curline = sr.ReadLine().ToString();

					// curline is not empty, let's parse
					results = curline.Split(delims);

					if (results[0].IndexOf("Vertices") < 0)
					{
						throw new Exception("Couldn't parse out '*Vertices'.");
					} 
					else 
					{
						State.Graph = null;

						for (int i = 1; i < results.Length; i++)
						{
							try
							{
								if (Int32.Parse(results[i]) > 0)
								{
									State.Graph = new ChemGraph(Int32.Parse(results[i]), this);
									Trace.Write("Parse Info", "Expect graph to have " + Int32.Parse(results[i]) + " vertices.");
									break;
								}
							}
							catch (FormatException fe)
							{
								Trace.Warn("Error in for loop line 200: " + fe.Message);
								continue;
							}
						}
			
						if (State.Graph == null)
						{
							throw new Exception("Parse error: Couldn't find the number of vertices to parse.");
						}
						
					}

					// We want curline to already have the first line in it at beg. of phase 2
					curline = sr.ReadLine().ToString();

					//
					// PHASE 2: Parse out vertices
					//
					// While there are still vertices to parse
					while (curline.IndexOf("Edges") < 0) 
					{
						// get rid of whitespace
						while (curline.Trim() == "") curline = sr.ReadLine().ToString();

						StringBuilder vName = new StringBuilder("");
						string sName;
						results = curline.Split(delims);
					
						int iter = 0;
						foreach (string s in results) 
						{
							if (iter != 0) 
							{
								vName.Append(s);
								iter++;
							}

							iter++;
						}

						sName = vName.ToString().Replace("\"", "");

						Regex r = new Regex(chemRegex.Text);
						if (r.IsMatch(sName)) 
						{
							// CHEMICAL
							//Trace.Write("IsChemical", sName);
							State.Graph.addVertex(Int32.Parse(results[0]), sName, true);
						} 
						else 
						{
							// ASSAY
							//Trace.Write("IsAssay", sName);
							State.Graph.addVertex(Int32.Parse(results[0]), sName, false);
						}
						
						curline = sr.ReadLine().ToString();
					}

					Trace.Write("Parse Info", "Graph now has " + State.Graph.NumVertices + " actual, parsed vertices.");

					//
					// PHASE 3: Parse out *Edges
					//
					// get rid of whitespace
					while (curline.Trim() == "") curline = sr.ReadLine().ToString();

					if (curline.IndexOf("Edges") < 0) 
					{
						throw new Exception("Couldn't parse out '*Edges'.");
					} 
					else 
					{
						// continue
						Trace.Write("Parse Info", "Beginning parse of edge data...");
					}

					//
					// PHASE 4: Parse out edges
					//
					while(sr.Peek() >= 0) 
					{
						curline = sr.ReadLine().ToString();

						// get rid of whitespace
						if (curline.Trim() == "") 
						{
							continue;
						}

						results = curline.Split(delims);

						//Trace.Warn("", Int32.Parse(results[0]) + " -- " + Int32.Parse(results[1]));
						State.Graph.addEdge(Int32.Parse(results[0]), Int32.Parse(results[1]), "");
					}

					Trace.Write("Parse Info", "Graph now has " + State.Graph.NumEdges + " actual, parsed edges.");
					Trace.Write("Parsing complete...", "");

					//
					// Optionally change graph to be non-bipartite -- ONLY FOR UPLOAD CASE
					//
					if (chemchem.Checked == true) 
					{
						chemChemGraph();
					}
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to parse, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Parse failure: " + exp.Message;
					return;
				}
			}
			else if (inputNodeEdges.Checked == true)
			{
				/*
				 * File of the form:
				 * 
				 * N0 : N1 N2 N3
				 * N1 : N7
				 * N9 : N3 N4
				 * 
				 * NOTE that if something on the right-hand side isn't found on the left-hand side, it's ignored
				 * 
				 */

				try
				{
					StreamReader sr = new StreamReader(inputNodeEdgesUpload.PostedFile.InputStream);

					string[] results = null; // to store the results of the split
					char[] delims = new char[] {' ','\t'};

					Trace.Write("Parsing beginning...", "");

					//
					// PHASE 1: Parse out vertices
					//
					// While there are still vertices to parse
					string curline;
					ArrayList vertices = new ArrayList();
					while (sr.Peek() >= 0) 
					{
						// Read in the current line (the line AFTER the position of the cursor)
						curline = sr.ReadLine().ToString();

						// If this line is empty, move on
						if (curline.Trim() == "") continue;

						curline.Replace("\t", " ");
						curline.Replace("  ", " ");

						StringBuilder vName = new StringBuilder("");
						results = curline.Split(delims);
					
						foreach (string s in results) 
						{
							if (s == ":") break;
							vName.Append(s);
						}
						
						vertices.Add(vName.Replace(" ", "").ToString());
					}

					State.Graph = new ChemGraph(vertices.Count, this);
 
					int id = 0;
					foreach (string s in vertices)
					{
						State.Graph.addVertex(id, s, false);
						id++;
					}

					//
					// PHASE 2: Parse out edges
					//

					Trace.Write("Phase 2 beginning...", "");
					
					// re-open the stream
					inputNodeEdgesUpload.PostedFile.InputStream.Seek(0, SeekOrigin.Begin);
					inputNodeEdgesUpload.PostedFile.InputStream.Position = 0;
					sr = new StreamReader(inputNodeEdgesUpload.PostedFile.InputStream);

					while (sr.Peek() >= 0) 
					{
						// Read in the current line (the line AFTER the position of the cursor)
						curline = sr.ReadLine().ToString();

						// If this line is empty, move on
						if (curline.Trim() == "") continue;

						curline.Replace("\t", " ");
						curline.Replace("  ", " ");

						Vertex curVert;
						StringBuilder vName = new StringBuilder("");
						string vNameString;
						ArrayList connectedTo = new ArrayList();
						results = curline.Split(delims);
						bool afterColon = false;
						foreach (string s in results) 
						{
							if (s == ":")
							{
								afterColon = true;

								vNameString = vName.Replace(" ", "").ToString();
								
								foreach (DictionaryEntry de in State.Graph.Vertices)
								{
									Vertex v = (Vertex)de.Value;
									if (v.Label == vNameString)
									{
										curVert = v;
									}
								}

								continue;
							}

							if (afterColon == false)
							{
								vName.Append(s);
								continue;
							}

							//
							// At this point we're after the colon, and we want to add to edges
							//
							if (s == " ") continue;

							foreach (DictionaryEntry de in State.Graph.Vertices)
							{
								Vertex v = (Vertex)de.Value;
								if (v.Label == s)
								{
									if (v != curVert) State.Graph.addEdge(curVert, v, "");
								}
							}

						}
					}

					Trace.Write("Parse Info", "Graph now has " + State.Graph.NumVertices + " actual, parsed vertices.");
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to parse, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Parse failure: " + exp.Message;
					return;
				}
			}
			else if (inputRandomER.Checked == true)
			{
				try
				{
					int numNodes = Int32.Parse(erNumNodes.Text);
					double p = double.Parse(erProbability.Text);

					if (numNodes < 2)
					{
						throw new Exception("Random E-R graph must have at least two vertices.");
					}

					State.Graph = new ChemGraph(numNodes, this);

					//
					// Add vertices
					//
					for (int i = 1; i <= numNodes; i++)
					{
						State.Graph.addVertex(i, "ER " + i.ToString(), false);
					}

					//
					// Add edges
					//
					double rand;
					for (int i = 1; i <= numNodes; i++)
					{
						for (int j = (i + 1); j <= numNodes; j++)
						{
							Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
							Random r = new Random((int)(i*j + numTicks));
							rand = r.NextDouble();
						
							if (rand <= p)
							{
								State.Graph.addEdge((Vertex)State.Graph.Vertices[i], (Vertex)State.Graph.Vertices[j], "");
							}
						}
					}
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to generate E-R graph, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "E-R Generation Failure: " + exp.Message;
					return;
				}
			}
			else if (inputRandomScaleFree.Checked == true)
			{
				//
				// TODO: Check whether this graph generation is correct.  
				// It seems weird, even though it fits the description in the paper.
				//

				try
				{
					int numNodes = Int32.Parse(sfNumNodes.Text);
					double m = double.Parse(sfM.Text);

					if (numNodes < 3)
					{
						throw new Exception("Random scale-free graph must have at least three vertices.");
					}

					if (m > 3)
					{
						throw new Exception("The value of m cannot be larger than the size of m0, which is 3.");
					}

					State.Graph = new ChemGraph(numNodes, this);

					//
					// Add m0 vertices -- a v-shape of 3 vertices
					//
					State.Graph.addVertex(0, "SF 0", false);
					State.Graph.addVertex(1, "SF 1", false);
					State.Graph.addVertex(2, "SF 2", false);
					State.Graph.addEdge(0, 1, "");
					State.Graph.addEdge(0, 2, "");

					//
					// Now add vertices
					//
					double totalDegree = 4; // the degree of m0
					double rand;
					for (int i = 3; i < numNodes; i++)
					{
						State.Graph.addVertex(i, "SF " + i.ToString(), false);

						// Loop through existing vertices
						for (int j = 0; j < i; j++)
						{
							double p = m * (((double)((Vertex)State.Graph.Vertices[j]).ConnectedTo.Count) / totalDegree);

							Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
							Random r = new Random((int)(i*j + numTicks));
							rand = r.NextDouble();

							if (rand <= p)
							{
								State.Graph.addEdge(i, j, "");
								totalDegree += 2;
							}
						}
					}
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to generate scale-free graph, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Scale-Free Generation Failure: " + exp.Message;
					return;
				}
			}
			else if (inputRandomScaleFreeRewiring.Checked == true)
			{
				try
				{

					double t = (double)Int32.Parse(sfrwT.Text);
					double pNewEdges = double.Parse(sfrwNewEdges.Text);
					double pRewire = double.Parse(sfrwRewire.Text);
					double m = double.Parse(sfrwM.Text);

					if (t < 3)
					{
						throw new Exception("Random scale-free re-wiring graph must have at least 3 time steps.");
					}

					if (m > 3)
					{
						throw new Exception("The value of m cannot be larger than the size of m0, which is 3.");
					}

					State.Graph = new ChemGraph((int)t, this);

					//
					// Add m0 vertices -- a v-shape of 3 vertices
					//
					State.Graph.addVertex(0, "SFRW 0", false);
					State.Graph.addVertex(1, "SFRW 1", false);
					State.Graph.addVertex(2, "SFRW 2", false);
					State.Graph.addEdge(0, 1, "");
					State.Graph.addEdge(0, 2, "");
					int nextID = 3;
				
					// Run for t time steps
					double rand;
					double p;
					double totalDegree = 4;
					Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
					Random r = new Random((int)(numTicks));
					for (int i = 0; i < t; i++)
					{
						rand = r.NextDouble();

						if (rand <= pNewEdges)
						{
							//
							// Add m new edges
							//
							for (int j = 0; j < m; j++)
							{
								// Choose a random first vertex
								Vertex v1 = (Vertex)State.Graph.Vertices[r.Next(State.Graph.Vertices.Count)];
							
								// Connect to a (preferred) random vertex
								foreach (DictionaryEntry de in State.Graph.Vertices)
								{
									Vertex v2 = (Vertex)de.Value;

									if (v1 == v2) continue;

									p = (((double)v2.ConnectedTo.Count + 1.0) / (totalDegree + State.Graph.Vertices.Count));
									rand = r.NextDouble();

									if (rand <= p)
									{
										State.Graph.addEdge(v1, v2, "");
										totalDegree += 2;
									}
								}
							}
						}
						else if ((rand > pNewEdges) && (rand <= (pNewEdges + pRewire)))
						{
							//
							// Rewire m times
							//
							for (int j = 0; j < m; j++)
							{
								// Choose a random first vertex
								Vertex v1 = (Vertex)State.Graph.Vertices[r.Next(State.Graph.Vertices.Count)];

								// Delete a random edge from it, if it's connected
								if (v1.ConnectedTo.Count > 0)
								{
									bool deletedP = false;
									while (deletedP == false)
									{
										foreach (Vertex v2 in v1.ConnectedTo)
										{
											p = 1.0 / v1.ConnectedTo.Count;
											rand = r.NextDouble();

											if (rand <= p)
											{
												State.Graph.removeEdge(v1, v2);
												totalDegree -= 2;
												deletedP = true;
												break;
											}
										}
									}
								}

								// Connect to a (preferred) random vertex
								foreach (DictionaryEntry de in State.Graph.Vertices)
								{
									Vertex v2 = (Vertex)de.Value;

									if (v1 == v2) continue;

									p = (((double)v2.ConnectedTo.Count + 1.0) / (totalDegree + State.Graph.Vertices.Count));
									rand = r.NextDouble();

									if (rand <= p)
									{
										State.Graph.addEdge(v1, v2, "");
										totalDegree += 2;
									}
								}
							}
						}
						else
						{
							//
							// Add a new node
							//

							State.Graph.addVertex(nextID, "SFRW " + nextID.ToString(), false);

							// Loop through existing vertices
							for (int j = 0; j < nextID; j++)
							{
								p = (((double)((Vertex)State.Graph.Vertices[j]).ConnectedTo.Count) / totalDegree);
								rand = r.NextDouble();

								if (rand <= p)
								{
									State.Graph.addEdge(nextID, j, "");
									totalDegree += 2;
								}
							}

							nextID++;
						}
					}
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to generate scale-free re-wiring graph, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Scale-Free Re-wiring Generation Failure: " + exp.Message;
					return;
				}
			}
			else if (inputRandomHierarchical.Checked == true)
			{
				try
				{
					int cs = Int32.Parse(hierCS.Text);
					int n = Int32.Parse(hierN.Text);
					int nodeNum = 0;

					ChemGraph curgraph = new ChemGraph(cs, this);
					Vertex center = new Vertex(nodeNum, "HIER " + nodeNum.ToString(), false);
					curgraph.addVertex(center);
					nodeNum++;

					//
					// Construct g0
					//
					for (int i = 0; i < (cs - 1); i++)
					{
						curgraph.addVertex(nodeNum, "HIER " + nodeNum.ToString(), false);
						curgraph.getVertex(nodeNum).IsPeripheral = true; //make peripheral nodes peripheral
					
						for (int j = 0; j < nodeNum; j++)
						{
							curgraph.addEdge(nodeNum, j, "");
						}
					
						nodeNum++;
					}

					//
					// Iteratively add hierarchy n times
					//
					for (int i = 0; i < n; i++)
					{
						ChemGraph nextgraph = new ChemGraph(curgraph.NumVertices * cs, this);
						ArrayList oldVertices = new ArrayList(curgraph.NumVertices);

						//
						// Make one copy of the old graph, putting vertices into oldVertices
						// to signify that this is the CORE GROUP of this new hierarchy level
						// 
						// We will later take every vertex in the core group and de-peripheral it
						// to signify that we don't need to connect it to the center vertex
						//
						for (int curID = 0; curID < nodeNum; curID++)
						{
							oldVertices.Add(curgraph.getVertex(curID));
							nextgraph.addVertex(curgraph.getVertex(curID));
							
							foreach (Edge ed in curgraph.getVertex(curID).InEdges)
							{
								nextgraph.addEdge(ed.v1, ed.v2, "");
							}
						}

						//
						// Make cs - 1 copies of the old graph -- these will connect to the center
						//
						for (int j = 0; j < (cs - 1); j++)
						{
							Hashtable ht = new Hashtable(oldVertices.Count);

							// Copy the vertices into the new group, matching their old with their new IDs
							foreach (Vertex v in oldVertices)
							{
								Vertex newVert = new Vertex(nodeNum, "HIER " + nodeNum.ToString(), false);
								newVert.IsPeripheral = v.IsPeripheral;
								nextgraph.addVertex(newVert);
								ht.Add(v.ID, nodeNum);
								nodeNum++;
							}

							// Copy the edges they were in now, using hashtable pairing information
							foreach (Vertex old in oldVertices)
							{
								foreach (Vertex oldConn in old.ConnectedTo)
								{
									Vertex nv1 = nextgraph.getVertex((int)ht[old.ID]);
									Vertex nv2 = nextgraph.getVertex((int)ht[oldConn.ID]);
									nextgraph.addEdge(nv1, nv2, "");
								}
							}
						}

						//
						// Connect peripheral nodes to the center
						//
						// First wipe out peripherals in core group
						foreach (Vertex v in oldVertices)
						{
							v.IsPeripheral = false;
						}

						// Now connect peripherals to center node
						foreach (DictionaryEntry de in nextgraph.Vertices)
						{
							Vertex v = (Vertex)de.Value;

							if (v.IsPeripheral == true)
							{
								nextgraph.addEdge(v, center, "");
							}
						}

						curgraph = nextgraph;
					}

					State.Graph = curgraph;
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to generate hierarchical graph, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Hierarchical Generation Failure: " + exp.Message;
					return;
				}
			}
			else if (inputRandomSmallWorld.Checked == true)
			{
				try
				{
					int n = Int32.Parse(swN.Text);
					int k = Int32.Parse(swK.Text);
					double p = double.Parse(swP.Text);

					if ((k % 2) != 0)
					{
						throw new Exception("Please choose an even k (number of neighbors to connect to).");
					}

					// Create the graph
					State.Graph = new ChemGraph(n, this);

					//
					// Create vertices
					//
					for (int i = 0; i < n; i++)
					{
						State.Graph.addVertex(i, "SW " + i.ToString(), false);
					}

					//
					// Connect vertices
					//
					for (int i = 0; i < n; i++)
					{
						for (int j = (-1)*(k/2); j < 0; j++)
						{
							State.Graph.addEdge(i, (i + j + n) % n, "");
						}

						for (int j = 1; j <= (k/2); j++)
						{
							State.Graph.addEdge(i, (i + j + n) % n, "");
						}
					}

					//
					// Randomly re-wire each edge
					//
					Int64 numTicks = System.DateTime.Now.Ticks % 1000000;;
					Random r = new Random((int)(numTicks));;
					double rand;
					
					ArrayList edges = new ArrayList(State.Graph.Edges.Count);
					foreach (Edge ed in State.Graph.Edges)
					{
						edges.Add(ed);
					}

					foreach (Edge ed in edges)
					{
						rand = r.NextDouble();

						if (rand <= p)
						{
							//
							// Re-wire this edge
							//
							Vertex v1 = ed.v1;

							int newID = r.Next(State.Graph.NumVertices);
							while (newID == v1.ID)
							{
								newID = r.Next(State.Graph.NumVertices);
							}

							Vertex v2 = State.Graph.getVertex(newID);

							State.Graph.removeEdge(ed.v1, ed.v2);

							State.Graph.addEdge(v1, v2, "");
						}
					}
				}
				catch (Exception exp)
				{
					Trace.Warn("Failed to generate small-world graph, error: " + exp);
					State.Graph = null;
					parseError.Visible = true;
					lblError.Text = "Small-world Generation Failure: " + exp.Message;
					return;
				}
			}
			else
			{
				throw new Exception("wtf kind of graph are you asking for?");
			}

			//
			// Calculate component info (lots of it)
			//
			State.Graph.UpdateComponents();		

			//
			// Continuing on...populating vertex data into listboxes (edge data is populated upon vertex selection)
			//
			verticesLB.Rows = 11;
			edgesLB.Rows = 11;

			//
			// populate vertex info
			//
			foreach(DictionaryEntry de in State.Graph.Vertices)
			{
				State.SelectedVertex = (Vertex)de.Value;
				break;
			}
			updateVertexListBox();

			//
			// Draw the 2D Fruchterman/Reingold graph
			//
			if (doFR.Checked)
			{
				frGraph(State.Graph); // Calculates new vertex positions
			}
			else
			{
				setupGraph(State.Graph);
			}

			//
			// Hide normal graph link/laplacian graph display (and maybe the LLE link)
			//
			State.Graph.DisplayType = displayType.Normal;

			//
			// Change the vertex display
			//
			changeVertexToCurrent(); // Will draw graph

			// Successfull completion, open up display
			importFile.Visible = false;
			analysisDisplay.Visible = true;
		}
		
		/// <summary>
		/// Updates the vertices listbox
		/// </summary>
		public void updateVertexListBox()
		{
			verticesLB.Items.Clear();
			collapseLB.Items.Clear();

			int maxLen = 0;
			foreach(DictionaryEntry de in State.Graph.Vertices) 
			{
				Vertex v = (Vertex)de.Value;
				verticesLB.Items.Add(new ListItem(v.Label, v.ID.ToString()));
				collapseLB.Items.Add(new ListItem(v.Label, v.ID.ToString()));
				
				int myLen = ((Vertex)(de.Value)).Label.Length;
				if (myLen > maxLen) 
				{
					maxLen = myLen;
				}
			}

			verticesLB.Items[0].Selected = true;
			collapseLB.Items[0].Selected = true;
					
			if (maxLen < 21) maxLen = 21;
			verticesLB.Width = maxLen * 8;
			collapseLB.Width = maxLen * 8;
			edgesLB.Width = maxLen * 8;
			pnlVertex.Width = maxLen * 18;
		}

		/// <summary>
		/// Processes a change in node selection from a list box
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Vertex_Select(Object sender, EventArgs e)
		{
			State.SelectedVertex = State.Graph.getVertex(Int32.Parse(((ListBox)sender).SelectedValue));
			changeVertexToCurrent();
		}

		/// <summary>
		/// Processes a button click signifying a desire to collapse vertices
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Collapse_Vertices(Object sender, EventArgs e)
		{
			if ((newNodeName.Text == "") || (newNodeName.Text == "Please enter a new node name!") ||
				(newNodeName.Text == "Please choose at least two nodes!"))
			{
				newNodeName.Text = "Please enter a new node name!";
				newNodeName.ForeColor = Color.Red;
				return;
			}

			//
			// Collapse
			//
			ArrayList nodes = new ArrayList();

			foreach(ListItem li in collapseLB.Items)
			{
				if (li.Selected == true)
				{
					nodes.Add(State.Graph.getVertex(Int32.Parse(li.Value)));
				}
			}

			if (nodes.Count < 2)
			{
				newNodeName.Text = "Please choose at least two nodes!";
				newNodeName.ForeColor = Color.Red;
				return;
			}

			Vertex newnode = State.Graph.collapseNodes(newNodeName.Text, nodes);			

			//
			// Set currentnode to new node
			//
			State.SelectedVertex = newnode;

			Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
			Random r = new Random((int)numTicks);
			foreach(DictionaryEntry de in State.Graph.Vertices)
			{
				Vertex v = (Vertex)de.Value;
				v.X = r.NextDouble();
				v.Y = r.NextDouble();
			}

			if (doFR.Checked == true)
			{
				frGraph(State.Graph);
				updateVertexListBox();
			}
			else
			{
				setupGraph(State.Graph);
				updateVertexListBox();
			}

			// Redraw screen focusing on new node
			changeVertexToCurrent();

			pnlCollapse.Visible = false;
			analysisDisplay.Visible = true;
		}

		/// <summary>
		/// Processes a button click signifying a desire to show the collapse vertices panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Collapse_Panel(Object sender, EventArgs e)
		{
			analysisDisplay.Visible = false;
			pnlCollapse.Visible = true;
		}

		/// <summary>
		/// Processes a button click signifying a desire to show the text output panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void TextOutput_Panel(Object sender, EventArgs e)
		{
			analysisDisplay.Visible = false;
			pnlTextOutput.Visible = true;
		}


		/// <summary>
		/// Processes a button click signifying a desire to show the collapse vertices panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Cancel_Collapse(Object sender, EventArgs e)
		{
			analysisDisplay.Visible = true;
			pnlCollapse.Visible = false;

			newNodeName.Text = "";
			collapseLB.ClearSelection();
			collapseLB.Items[0].Selected = true;
		}

		/// <summary>
		/// Processes a request to export data to a text file
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Click_Export(Object sender, EventArgs e)
		{
			if ((outNode.Checked == false) && (outAdjEigenvalueInfo.Checked == false) &&
				(outLapEigenvalueInfo.Checked == false) && (outNormLapEigenvalueInfo.Checked == false) &&
				(outAdjMat.Checked == false) && (outLapMat.Checked == false) && (outNormLapMat.Checked == false))
			{
				return;
			}

			Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
			Random r = new Random((int)numTicks);

			if (outNode.Checked == true)
			{
				string shortname = "node" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				StringBuilder sb = new StringBuilder("Label,ID,");
				if (outNodeDegree.Checked == true) sb.Append("Degree,");
				if (outNodeCK.Checked == true) sb.Append("ClusterCoeff,");
				if (outNodeAvgDist.Checked == true) sb.Append("AvgDist,");
				if (outNodeMaxDist.Checked == true) sb.Append("MaxDist,");

				if (outNodeAdjEig.Checked == true) sb.Append("AdjEigVec0,");
				if (outNodeAdjEig.Checked == true) sb.Append("AdjEigVec1,");
				if (outNodeAdjEig.Checked == true) sb.Append("AdjEigVec2,");
				if (outNodeAdjEig.Checked == true) sb.Append("AdjEigVec3,");

				if (outNodeLapEig.Checked == true) sb.Append("LapEigVec0,");
				if (outNodeLapEig.Checked == true) sb.Append("LapEigVec1,");
				if (outNodeLapEig.Checked == true) sb.Append("LapEigVec2,");
				if (outNodeLapEig.Checked == true) sb.Append("LapEigVec3,");

				if (outNodeNormLapEig.Checked == true) sb.Append("NormLapEigVec0,");
				if (outNodeNormLapEig.Checked == true) sb.Append("NormLapEigVec1,");
				if (outNodeNormLapEig.Checked == true) sb.Append("NormLapEigVec2,");
				if (outNodeNormLapEig.Checked == true) sb.Append("NormLapEigVec3,");

				if (outNodeLLE.Checked == true) sb.Append("LapLLEx,");
				if (outNodeLLE.Checked == true) sb.Append("LapLLEy,");
				if (outNodeLLE.Checked == true) sb.Append("LapLLEz,");

				sb.Remove(sb.Length - 1, 1);

				sw.WriteLine(sb.ToString());

				foreach (DictionaryEntry de in State.SelectedVertex.ConnectedComponent.Vertices)
				{
					Vertex v = (Vertex)de.Value;
					StringBuilder line = new StringBuilder("'" + v.Label + "'" + "," + v.ID + ",");
					if (outNodeDegree.Checked == true) line.Append(v.ConnectedTo.Count).Append(",");
					if (outNodeCK.Checked == true) line.Append(v.ClusterCoeff).Append(",");
					if (outNodeAvgDist.Checked == true) line.Append(v.AvgDist).Append(",");
					if (outNodeMaxDist.Checked == true) line.Append(v.MaxDist).Append(",");

					if (outNodeAdjEig.Checked == true) line.Append(v.getAdjacencyEigenvalue(0)).Append(",");
					if (outNodeAdjEig.Checked == true) line.Append(v.getAdjacencyEigenvalue(1)).Append(",");
					if (outNodeAdjEig.Checked == true) line.Append(v.getAdjacencyEigenvalue(2)).Append(",");
					if (outNodeAdjEig.Checked == true) line.Append(v.getAdjacencyEigenvalue(3)).Append(",");

					if (outNodeLapEig.Checked == true) line.Append(v.getLaplacianEigenvalue(0)).Append(",");
					if (outNodeLapEig.Checked == true) line.Append(v.getLaplacianEigenvalue(1)).Append(",");
					if (outNodeLapEig.Checked == true) line.Append(v.getLaplacianEigenvalue(2)).Append(",");
					if (outNodeLapEig.Checked == true) line.Append(v.getLaplacianEigenvalue(3)).Append(",");

					if (outNodeNormLapEig.Checked == true) line.Append(v.getNormalizedLaplacianEigenvalue(0)).Append(",");
					if (outNodeNormLapEig.Checked == true) line.Append(v.getNormalizedLaplacianEigenvalue(1)).Append(",");
					if (outNodeNormLapEig.Checked == true) line.Append(v.getNormalizedLaplacianEigenvalue(2)).Append(",");
					if (outNodeNormLapEig.Checked == true) line.Append(v.getNormalizedLaplacianEigenvalue(3)).Append(",");

					if (outNodeLLE.Checked == true)
					{
						double val = 0.0;
						if (v.getLaplacianEigenvalue(1) > 0)
						{
							val = Math.Log10(v.getLaplacianEigenvalue(1));
						}
						else if (v.getLaplacianEigenvalue(1) == 0)
						{
							val = 0.0;
						}
						else // < 0
						{
							val = -1 * Math.Log10(-1 * v.getLaplacianEigenvalue(1));
						}
						line.Append(val.ToString() + ",");
						
						val = 0.0;
						if (v.getLaplacianEigenvalue(2) > 0)
						{
							val = Math.Log10(v.getLaplacianEigenvalue(2));
						}
						else if (v.getLaplacianEigenvalue(2) == 0)
						{
							val = 0.0;
						}
						else // < 0
						{
							val = -1 * Math.Log10(-1 * v.getLaplacianEigenvalue(2));
						}
						line.Append(val.ToString() + ",");

						val = 0.0;
						if (v.getLaplacianEigenvalue(3) > 0)
						{
							val = Math.Log10(v.getLaplacianEigenvalue(3));
						}
						else if (v.getLaplacianEigenvalue(3) == 0)
						{
							val = 0.0;
						}
						else // < 0
						{
							val = -1 * Math.Log10(-1 * v.getLaplacianEigenvalue(3));
						}
						line.Append(val.ToString() + ",");
					}

					line.Remove(line.Length - 1, 1);

					sw.WriteLine(line.ToString());
				}
				
				sw.Close();

				linkNodeInfo.NavigateUrl = "export/" + shortname;
				linkNodeInfo.Visible = true;
			}
			else
			{
				linkNodeInfo.Visible = false;
			}

			if (outAdjEigenvalueInfo.Checked == true)
			{
				string shortname = "adjeigen" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				//
				// Header line
				//
				StringBuilder sb = new StringBuilder("Eigenvalue,");
				if (outAdjIVP.Checked == true) sb.Append("IVP").Append(",");
				if (outAdjSpec.Checked == true) sb.Append("lambda/Sqrt(Np(1-p)),pSqrt(Np(1-p)),");
				if (outAdjEigenvector.Checked == true)
				{
					for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
					{
						sb.Append("VecComponent" + i.ToString() + "(" 
							+ ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + ")" + ",");
					}
				}
				sb.Remove(sb.Length - 1, 1);
				sw.WriteLine(sb.ToString());

				//
				// Data
				//
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					double myEigen = State.SelectedVertex.ConnectedComponent.adjEigens[i];
					StringBuilder line = new StringBuilder(myEigen.ToString() + ",");
					if (outAdjIVP.Checked == true) line.Append(State.SelectedVertex.ConnectedComponent.adjIVP[i] + ",");
					if (outAdjSpec.Checked == true)
					{
						double p, result, root, x, y;
						int err;

						p = 0.0;
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							Dirac.delta(10.0, (myEigen - State.SelectedVertex.ConnectedComponent.adjEigens[j]), 
								0.000000001, out result, out err);

							if (err != 0)
							{
								throw new Exception("Click_Export (adj. eigen): Dirac.delta() returned error code " + err.ToString());
							}

							if (Math.Abs(result) > 0.0000000001)
							{
								p += (1.0 / ((double)State.SelectedVertex.ConnectedComponent.NumVertices)) * result;
							}
						}

						root = Math.Sqrt(Math.Abs(((double)State.SelectedVertex.ConnectedComponent.NumVertices) * p * (1.0 - p)));

						x = myEigen/root;
						y = p * root;

						line.Append(x + "," + y + ",");
					}
					if (outAdjEigenvector.Checked == true)
					{
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							line.Append(((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(j)).getAdjacencyEigenvalue(i).ToString() + ",");
						}
					}
					line.Remove(line.Length - 1, 1);
					sw.WriteLine(line.ToString());
				}

				//
				// Clean-up
				//
				sw.Close();

				linkAdjEigenvalueInfo.NavigateUrl = "export/" + shortname;
				linkAdjEigenvalueInfo.Visible = true;
			}
			else
			{
				linkAdjEigenvalueInfo.Visible = false;
			}

			if (outLapEigenvalueInfo.Checked == true)
			{
				string shortname = "lapeigen" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				//
				// Header line
				//
				StringBuilder sb = new StringBuilder("Eigenvalue,");
				if (outLapIVP.Checked == true) sb.Append("IVP").Append(",");
				if (outLapSpec.Checked == true) sb.Append("lambda/Sqrt(Np(1-p)),pSqrt(Np(1-p)),");
				if (outLapEigenvector.Checked == true)
				{
					for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
					{
						sb.Append("VecComponent" + i.ToString() + "(" 
							+ ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + ")" + ",");
					}
				}
				sb.Remove(sb.Length - 1, 1);
				sw.WriteLine(sb.ToString());

				//
				// Data
				//
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					double myEigen = State.SelectedVertex.ConnectedComponent.lapEigens[i];
					StringBuilder line = new StringBuilder(myEigen.ToString() + ",");
					if (outLapIVP.Checked == true) line.Append(State.SelectedVertex.ConnectedComponent.lapIVP[i] + ",");
					if (outLapSpec.Checked == true)
					{
						double p, result, root, x, y;
						int err;

						p = 0.0;
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							Dirac.delta(10.0, (myEigen - State.SelectedVertex.ConnectedComponent.lapEigens[j]), 
								0.000000001, out result, out err);

							if (err != 0)
							{
								throw new Exception("Click_Export (lap. eigen): Dirac.delta() returned error code " + err.ToString());
							}

							if (Math.Abs(result) > 0.0000000001)
							{
								p += (1.0 / ((double)State.SelectedVertex.ConnectedComponent.NumVertices)) * result;
							}
						}

						root = Math.Sqrt(Math.Abs(((double)State.SelectedVertex.ConnectedComponent.NumVertices) * p * (1.0 - p)));

						x = myEigen/root;
						y = p * root;

						line.Append(x + "," + y + ",");
					}
					if (outLapEigenvector.Checked == true)
					{
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							line.Append(((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(j)).getLaplacianEigenvalue(i).ToString() + ",");
						}
					}
					line.Remove(line.Length - 1, 1);
					sw.WriteLine(line.ToString());
				}

				//
				// Clean-up
				//
				sw.Close();

				linkLapEigenvalueInfo.NavigateUrl = "export/" + shortname;
				linkLapEigenvalueInfo.Visible = true;
			}
			else
			{
				linkLapEigenvalueInfo.Visible = false;
			}

			if (outNormLapEigenvalueInfo.Checked == true)
			{
				string shortname = "normlapeigen" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				//
				// Header line
				//
				StringBuilder sb = new StringBuilder("Eigenvalue,");
				if (outNormLapIVP.Checked == true) sb.Append("IVP").Append(",");
				if (outNormLapSpec.Checked == true) sb.Append("lambda/Sqrt(Np(1-p)),pSqrt(Np(1-p)),");
				if (outNormLapEigenvector.Checked == true)
				{
					for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
					{
						sb.Append("VecComponent" + i.ToString() + "(" 
							+ ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + ")" + ",");
					}
				}
				sb.Remove(sb.Length - 1, 1);
				sw.WriteLine(sb.ToString());

				//
				// Data
				//
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					double myEigen = State.SelectedVertex.ConnectedComponent.normLapEigens[i];
					StringBuilder line = new StringBuilder(myEigen.ToString() + ",");
					if (outNormLapIVP.Checked == true) line.Append(State.SelectedVertex.ConnectedComponent.normLapIVP[i] + ",");
					if (outNormLapSpec.Checked == true)
					{
						double p, result, root, x, y;
						int err;

						p = 0.0;
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							Dirac.delta(10.0, (myEigen - State.SelectedVertex.ConnectedComponent.normLapEigens[j]), 
								0.000000001, out result, out err);

							if (err != 0)
							{
								throw new Exception("Click_Export (norm. lap. eigen): Dirac.delta() returned error code " + err.ToString());
							}

							if (Math.Abs(result) > 0.0000000001)
							{
								p += (1.0 / ((double)State.SelectedVertex.ConnectedComponent.NumVertices)) * result;
							}
						}

						root = Math.Sqrt(Math.Abs(((double)State.SelectedVertex.ConnectedComponent.NumVertices) * p * (1.0 - p)));

						x = myEigen/root;
						y = p * root;

						line.Append(x + "," + y + ",");
					}
					if (outNormLapEigenvector.Checked == true)
					{
						for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++)
						{
							line.Append(((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(j)).getNormalizedLaplacianEigenvalue(i).ToString() + ",");
						}
					}
					line.Remove(line.Length - 1, 1);
					sw.WriteLine(line.ToString());
				}

				//
				// Clean-up
				//
				sw.Close();

				linkNormLapEigenvalueInfo.NavigateUrl = "export/" + shortname;
				linkNormLapEigenvalueInfo.Visible = true;
			}
			else
			{
				linkNormLapEigenvalueInfo.Visible = false;
			}

			if (outAdjMat.Checked == true)
			{
				string shortname = "adjmat" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				StringBuilder line = new StringBuilder(",");;
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					line.Append("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");
				}
				line.Remove(line.Length - 1, 1);
				sw.WriteLine(line.ToString());

				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++) 
				{
					line = new StringBuilder("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");

					for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++) 
					{
						line.Append(State.SelectedVertex.ConnectedComponent.adj.getValue(i, j) + ",");
					}
			
					line.Remove(line.Length - 1, 1);

					sw.WriteLine(line.ToString());
				}
				
				sw.Close();

				linkAdjMat.NavigateUrl = "export/" + shortname;
				linkAdjMat.Visible = true;
			}
			else
			{
				linkAdjMat.Visible = false;
			}

			if (outLapMat.Checked == true)
			{
				string shortname = "lapmat" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);
				
				StringBuilder line = new StringBuilder(",");;
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					line.Append("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");
				}
				line.Remove(line.Length - 1, 1);
				sw.WriteLine(line.ToString());

				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++) 
				{
					line = new StringBuilder("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");

					for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++) 
					{
						Vertex v = (Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i);
						Vertex u = (Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(j);

						double val = 0.0;
						if (i == j) 
						{
							val = v.ConnectedTo.Count;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							val = -1.0;
						} 
						else 
						{
							val = 0;
						}

						line.Append(val.ToString() + ",");
					}
			
					line.Remove(line.Length - 1, 1);

					sw.WriteLine(line.ToString());
				}
				
				sw.Close();

				linkLapMat.NavigateUrl = "export/" + shortname;
				linkLapMat.Visible = true;
			}
			else
			{
				linkLapMat.Visible = false;
			}

			if (outNormLapMat.Checked == true)
			{
				string shortname = "normlapmat" + r.Next() + ".csv";
				string fname = Server.MapPath("export\\" + shortname);
				StreamWriter sw = new StreamWriter(fname);
				State.Graph.addFile(fname);

				StringBuilder line = new StringBuilder(",");;
				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++)
				{
					line.Append("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");
				}
				line.Remove(line.Length - 1, 1);
				sw.WriteLine(line.ToString());

				for (int i = 0; i < State.SelectedVertex.ConnectedComponent.NumVertices; i++) 
				{
					line = new StringBuilder("'" + ((Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i)).Label + "',");

					for (int j = 0; j < State.SelectedVertex.ConnectedComponent.NumVertices; j++) 
					{
						Vertex v = (Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(i);
						Vertex u = (Vertex)State.SelectedVertex.ConnectedComponent.Vertices.GetByIndex(j);

						double val = 0.0;
						if (i == j) 
						{
							val = 1;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							val = - (1.0 / (Math.Sqrt(v.ConnectedTo.Count * u.ConnectedTo.Count)));
						} 
						else 
						{
							val = 0;
						}

						line.Append(val.ToString() + ",");
					}
			
					line.Remove(line.Length - 1, 1);

					sw.WriteLine(line.ToString());
				}
				
				sw.Close();

				linkNormLapMat.NavigateUrl = "export/" + shortname;
				linkNormLapMat.Visible = true;
			}
			else
			{
				linkNormLapMat.Visible = false;
			}

			// Now make the links visible
			pnlExportLinks.Visible = true;
			pnlTextOutput.Visible = false;
		}

		/// <summary>
		/// Processes a click on the button that signifies that the user is finished with the text export feature.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Click_DoneExport(Object sender, EventArgs e)
		{
			pnlExportLinks.Visible = false;
			analysisDisplay.Visible = true;
		}

		/// <summary>
		/// Processes an unchecking of the node checkbox in the export node panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Unclick_outNode(Object sender, EventArgs e)
		{
			if (outNode.Checked == false)
			{
				outNodeDegree.Enabled = false;
				outNodeCK.Enabled = false;
				outNodeAvgDist.Enabled = false;
				outNodeMaxDist.Enabled = false;
				outNodeAdjEig.Enabled = false;
				outNodeLapEig.Enabled = false;
				outNodeNormLapEig.Enabled = false;
				outNodeLLE.Enabled = false;
				lblLabel.ForeColor = Color.Gray;
				lblIDNumber.ForeColor = Color.Gray;
			}
			else
			{
				outNodeDegree.Enabled = true;
				outNodeCK.Enabled = true;
				outNodeAvgDist.Enabled = true;
				outNodeMaxDist.Enabled = true;
				outNodeAdjEig.Enabled = true;
				outNodeLapEig.Enabled = true;
				outNodeNormLapEig.Enabled = true;
				outNodeLLE.Enabled = true;
				lblLabel.ForeColor = Color.Black;
				lblIDNumber.ForeColor = Color.Black;
			}
		}

		/// <summary>
		/// Processes an unchecking of the adjEigenvalue checkbox in the export panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Unclick_outAdjEigenvalueInfo(Object sender, EventArgs e)
		{
			if (outAdjEigenvalueInfo.Checked == false)
			{
				outAdjIVP.Enabled = false;
				outAdjSpec.Enabled = false;
				outAdjEigenvector.Enabled = false;
			}
			else
			{
				outAdjIVP.Enabled = true;
				outAdjSpec.Enabled = true;
				outAdjEigenvector.Enabled = true;
			}
		}

		/// <summary>
		/// Processes an unchecking of the lapEigenvalue checkbox in the export panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Unclick_outLapEigenvalueInfo(Object sender, EventArgs e)
		{
			if (outLapEigenvalueInfo.Checked == false)
			{
				outLapIVP.Enabled = false;
				outLapSpec.Enabled = false;
				outLapEigenvector.Enabled = false;
			}
			else
			{
				outLapIVP.Enabled = true;
				outLapSpec.Enabled = true;
				outLapEigenvector.Enabled = true;
			}
		}

		/// <summary>
		/// Processes an unchecking of the normLapEigenvalue checkbox in the export panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Unclick_outNormLapEigenvalueInfo(Object sender, EventArgs e)
		{
			if (outNormLapEigenvalueInfo.Checked == false)
			{
				outNormLapIVP.Enabled = false;
				outNormLapSpec.Enabled = false;
				outNormLapEigenvector.Enabled = false;
			}
			else
			{
				outNormLapIVP.Enabled = true;
				outNormLapSpec.Enabled = true;
				outNormLapEigenvector.Enabled = true;
			}
		}

		/// <summary>
		/// Processes a button click signifying a desire to show the collapse vertices panel
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void Cancel_TextOutput(Object sender, EventArgs e)
		{
			analysisDisplay.Visible = true;
			pnlTextOutput.Visible = false;
		}

		/// <summary>
		/// Take the current Chemical-Assay graph and make it a Chemical-Chemical graph
		/// </summary>
		private void chemChemGraph() {
			ChemGraph cg;
			ArrayList newVertices = new ArrayList();

			//
			// ADD VERTICES
			// Pull out the chemicals from the graph
			//
			foreach(DictionaryEntry de in State.Graph.Vertices) {
				Vertex v = (Vertex)(de.Value);
				
				// This graph only has chemical vertices, not assay vertices
				if (v.isChemical == true) {
					newVertices.Add(v);
				}
			}

			cg = new ChemGraph(newVertices.Count, this);
			cg.isBipartite = false;

			foreach(Vertex v in newVertices) {
				cg.addVertex(v.ID, v.Label, v.isChemical);
			}

			//
			// ADD EDGES -- the hard part
			//
			int target = Int32.Parse(numSimilarities.SelectedItem.Value);
			foreach(DictionaryEntry de in State.Graph.Vertices) { // NOTE: we use State.Graph.Vertices here, addEdge doesn't break 'cause it uses ID numbers, which are the same for cg and State.Graph
				Vertex v = (Vertex)(de.Value);

				if (v.isChemical == false) continue;

				foreach(DictionaryEntry de2 in State.Graph.Vertices) {
					Vertex u = (Vertex)(de2.Value);
					int i = 0;

					if (v == u) continue;

					// TODO: make this more efficient, .ConnectedTo doesn't need to recompute at every call
					foreach(Vertex uconn in u.ConnectedTo) {
						if (v.ConnectedTo.Contains(uconn)) {
							i++;
						}
					}

					if (i >= target) {
						cg.addEdge(v.ID, u.ID, "");
					}
				}
			}

			//
			// SWAP GRAPHS
			//
			State.Graph = cg;
		}


		/// <summary>
		///  Draws the Fruchterman/Reingold graph for a given ChemGraph
		/// </summary>
		/// <param name="g">The ChemGraph object on which to run the F-R algorithm.  Modifies g.v.X and g.v.Y values.</param>
		protected void frGraph(ChemGraph g) {
			//
			// Iteratively calculate new node positions
			//
			int area = 660*660;
			double k = Math.Sqrt(area / g.Vertices.Count);
			double t = 80;

			foreach(DictionaryEntry de in g.Vertices) {
				Vertex v = (Vertex)(de.Value);
				//Trace.Write(v.Label, "(" + v.X.ToString() + "," + v.Y.ToString() + ")");
			}

			while (t > 2) {
				//
				// Calculate repulsive forces
				//
				foreach(DictionaryEntry de in g.Vertices) {
					Vertex v = (Vertex)(de.Value);
					double delX = 0.0;
					double delY = 0.0;
					double delSize = 0.0;

					v.dispX = 0.0;
					v.dispY = 0.0;

					foreach(DictionaryEntry de2 in g.Vertices) {
						Vertex u = (Vertex)(de2.Value);

						if (u != v) {
							delX = v.X - u.X;
							delY = v.Y - u.Y;

							delSize = Math.Sqrt(delX*delX + delY*delY);
							v.dispX = v.dispX + (delX/delSize)*fr(delSize, k);
							v.dispY = v.dispY + (delY/delSize)*fr(delSize, k);
						}
					}

				}

				//
				// Calculate attractive forces
				//
				foreach(Edge e in g.Edges) {
					double delX = 0.0;
					double delY = 0.0;
					double delSize = 0.0;

					delX = e.v1.X - e.v2.X;
					delY = e.v1.Y - e.v2.Y;

					delSize = Math.Sqrt(delX*delX + delY*delY);

					e.v1.dispX = e.v1.dispX - (delX/delSize)*fa(delSize, k);
					e.v1.dispY = e.v1.dispY - (delY/delSize)*fa(delSize, k);

					e.v2.dispX = e.v2.dispX + (delX/delSize)*fa(delSize, k);
					e.v2.dispY = e.v2.dispY + (delY/delSize)*fa(delSize, k);
				}
		
				//
				// Limit displacement by temperature and prevent from being displaced outside frame
				//
				foreach(DictionaryEntry de in g.Vertices) {
					Vertex v = (Vertex)(de.Value);

					double sizeVDisp = Math.Sqrt(v.dispX*v.dispX + v.dispY*v.dispY);

					v.X = v.X + (v.dispX/sizeVDisp)*Math.Min(sizeVDisp, t);
					v.Y = v.Y + (v.dispY/sizeVDisp)*Math.Min(sizeVDisp, t);

					v.X = Math.Min(330, Math.Max(-330, v.X));
					v.Y = Math.Min(330, Math.Max(-330, v.Y));
				}


				if (t >= 20)
				{
					t = .80*t;	// May want to adjust this cooling function if the alg. takes too long
								// or the results could be improved
				}
				else if ((t < 25) && (t >=5))
				{
					t = .87*t;
				}
				else if (t < 5)
				{
					t = .95*t;
				}
			}
			
			foreach(DictionaryEntry de in g.Vertices) {
				Vertex v = (Vertex)(de.Value);
				//Trace.Write("FR " + v.Label, "(" + v.X.ToString() + "," + v.Y.ToString() + ")");
			}

			setupGraph(g); // scales node distances
		}


		/// <summary>
		/// Force attraction function
		/// </summary>
		/// <returns>Size of attraction</returns>
		private double fa(double x, double k) {
			return (x*x / k);
		}


		/// <summary>
		/// Force repelling function
		/// </summary>
		/// <returns>Size of repulsion</returns>
		private double fr(double x, double k) {
			return -(k*k / x);
		}


		/// <summary>
		///  Takes nodes with positions in all quadrants and stretches the graph to fit within the image
		/// </summary>
		private void setupGraph(ChemGraph g) {
			double xScale = 1.0;
			double yScale = 1.0;

			//
			// Scale graph
			//
			// Find x, y scales
			double minX = Double.MaxValue;
			double maxX = Double.MinValue;
			double minY = Double.MaxValue;
			double maxY = Double.MinValue;
			foreach(DictionaryEntry de in g.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v.X < minX) 
				{
					minX = v.X;
				}

				if (v.X > maxX) 
				{
					maxX = v.X;
				}

				if (v.Y < minY) 
				{
					minY = v.Y;
				}

				if (v.Y > maxY) 
				{
					maxY = v.Y;
				}
			}

			foreach(DictionaryEntry de in g.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.X -= (maxX + minX) / 2.0;
				v.Y -= (maxY + minY) / 2.0;
			}
				
			xScale = 330 / (maxX - ((maxX + minX) / 2.0));
			yScale = 330 / (maxY - ((maxY + minY) / 2.0));

			// Scale vertices
			foreach(DictionaryEntry de in g.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.X = v.X * xScale + 345;
				v.Y = v.Y * yScale + 345;
			}
		}


		/// <summary>
		/// Draws the main graph
		/// </summary>
		private void drawGraph() {
			//
			// TODO: make this a method of ChemGraph
			//

			int w = Int32.Parse(graphDisplay.Width.Value.ToString());
			int h = Int32.Parse(graphDisplay.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			Vertex selectedVertex = State.SelectedVertex;

			foreach(Edge e in State.Graph.Edges) {
				pad.DrawLine(dgPen, (int)e.v1.X, (int)e.v1.Y, (int)e.v2.X, (int)e.v2.Y);
			}

			//
			// Draw text labels
			//
			foreach(DictionaryEntry de in State.Graph.Vertices)
			{
				Vertex v = (Vertex)de.Value;

				if (v != selectedVertex)
				{
					pad.DrawString(v.Label, new Font(FontFamily.GenericSansSerif, 7, FontStyle.Regular), 
						darkGreenBrush, new PointF((float)(v.X + 0.0), (float)(v.Y + 0.0)));
				}
			}

			foreach(DictionaryEntry de in State.Graph.Vertices) {
				Vertex v = (Vertex)(de.Value);

				Trace.Write("drawGraph()", "Drawing " + v.Label);

				if (v == selectedVertex) {
					//pad.FillEllipse(redBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				} else if (selectedVertex.ConnectedComponent.Vertices.ContainsKey(v.ID)) {
					pad.FillEllipse(siennaBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				} else {
					pad.FillEllipse(blackBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				}
			}

			foreach(Edge e in selectedVertex.InEdges) {
				pad.DrawLine(orangePen, (int)e.v1.X, (int)e.v1.Y, (int)e.v2.X, (int)e.v2.Y);
			}

			foreach(DictionaryEntry de in State.Graph.Vertices) {
				Vertex v = (Vertex)(de.Value);

				Trace.Write("drawGraph()", "Drawing " + v.Label);

				if (selectedVertex.ConnectedTo.Contains(v)) {
					pad.FillEllipse(redBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				} else if (selectedVertex.ConnectedComponent.Vertices.ContainsKey(v.ID)) {
					//pad.FillEllipse(siennaBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				} else {
					//pad.FillEllipse(blackBrush, (int)v.X - 4, (int)v.Y - 4, 8, 8);
				}
			}

			// Now color selected vertex
			PointF[] points = new PointF[3];
			points[0] = new PointF((float)selectedVertex.X, (float)selectedVertex.Y - (float)(10.0f*Math.Sqrt(3)*(1.0f/3.0f)));
			points[1] = new PointF((float)selectedVertex.X - 7.0f, (float)selectedVertex.Y + (float)(5.0f*Math.Sqrt(3)));
			points[2] = new PointF((float)selectedVertex.X + 7.0f, (float)selectedVertex.Y + (float)(5.0f*Math.Sqrt(3)));
			pad.FillPolygon(redBrush, points);
			pad.DrawString(selectedVertex.Label, new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold), 
				darkGreenBrush, new PointF((float)(selectedVertex.X + 3.0), (float)(selectedVertex.Y - 9.0)));

			Random r = new Random();
			picName = "graph" + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			graphDisplay.ImageUrl = "graphs/" + picName;
		}


		/// <summary>
		/// Changes the large graph display between F-R and Laplacian LLE.
		/// </summary>
		/// <param name="sender">LinkButton of graph display type desired</param>
		/// <param name="e">Unused</param>
		protected void gDisplay_Change(object sender, EventArgs e)
		{
			if (sender == gNormal)
			{
				State.Graph.DisplayType = displayType.Normal;

				normalPanel.Visible = true;
				gNormal.Visible = false;

				laplacianPanel.Visible = false;
				gLaplacian.Visible = true;
			}
			else // sender == gLaplacian
			{
				State.Graph.DisplayType = displayType.LaplacianLLE;

				normalPanel.Visible = false;
				gNormal.Visible = true;

				laplacianPanel.Visible = true;
				gLaplacian.Visible = false;
			}
		}


		/// <summary>
		/// Processes a click on the main graph imagebuttons
		/// </summary>
		/// <param name="sender">The graphDisplay or laplacianDisplay imagebutton</param>
		/// <param name="e">Gives position of click to know which vertex to change to</param>
		protected void Image_Clicked(object sender, ImageClickEventArgs e) 
		{
			Vertex selected = State.SelectedVertex;

			foreach(Vertex v in selected.ConnectedTo) 
			{
				double dist = 0.0;
				if (sender == graphDisplay)
				{
					dist = Math.Sqrt((v.X - e.X)*(v.X - e.X) + (v.Y - e.Y)*(v.Y - e.Y));
				}
				else if (sender == laplacianDisplay)
				{
					dist = Math.Sqrt((v.XLap - e.X)*(v.XLap - e.X) + (v.YLap - e.Y)*(v.YLap - e.Y));
				}
				else
				{
					throw new Exception("Image_Clicked: I don't know what imagebutton was clicked.");
				}

				if (dist <= 4) 
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					return;
				}
			}

			
			foreach(DictionaryEntry de in selected.ConnectedComponent.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selected) continue;

				double dist = 0.0;
				if (sender == graphDisplay)
				{
					dist = Math.Sqrt((v.X - e.X)*(v.X - e.X) + (v.Y - e.Y)*(v.Y - e.Y));
				}
				else if (sender == laplacianDisplay)
				{
					dist = Math.Sqrt((v.XLap - e.X)*(v.XLap - e.X) + (v.YLap - e.Y)*(v.YLap - e.Y));
				}
				else
				{
					throw new Exception("Image_Clicked: I don't know what imagebutton was clicked.");
				}

				if (dist <= 4) 
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					return;
				}
			}

			foreach(DictionaryEntry de in State.Graph.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selected) continue;
				
				double dist = 0.0;
				if (sender == graphDisplay)
				{
					dist = Math.Sqrt((v.X - e.X)*(v.X - e.X) + (v.Y - e.Y)*(v.Y - e.Y));
				}
				else if (sender == laplacianDisplay)
				{
					dist = Math.Sqrt((v.XLap - e.X)*(v.XLap - e.X) + (v.YLap - e.Y)*(v.YLap - e.Y));
				}
				else
				{
					throw new Exception("Image_Clicked: I don't know what imagebutton was clicked.");
				}

				if (dist <= 4) 
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					return;
				}
			}

		}

		/// <summary>
		/// Draw the degree distribution of the currently active connected component
		/// </summary>
		private void drawDegreeDistribution() 
		{
			Vertex selectedVertex = State.SelectedVertex;
			int w = Int32.Parse(graphDegreeDistribution.Width.Value.ToString());
			int h = Int32.Parse(graphDegreeDistribution.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;
			int numBins = 0;
			double minDegree = selectedVertex.ConnectedComponent.MinDegree;
			double maxDegree = selectedVertex.ConnectedComponent.MaxDegree;

			int diff = (int)(maxDegree - minDegree);
			if (diff > 200)
			{
				numBins = 30;
			}
			else if ((diff > 100) && (diff <= 200)) 
			{
				numBins = 20;
			} 
			else if ((diff > 50) && (diff <= 100)) 
			{
				numBins = 15;
			} 
			else if ((diff > 30) && (diff <= 50)) 
			{
				numBins = 10;
			} 
			else if ((diff > 0) && (diff <= 30)) 
			{
				numBins = 5;
			}

			Trace.Write("drawDegreeDistribution", "minDegree = " + minDegree.ToString() + "; maxDegree = " + maxDegree.ToString());

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("k", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF((w / 2) - 3, h - 27));
			pad.DrawString("P(k)", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(1, (h / 2) - 5));

			pad.DrawString("0", new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, h - 38));
			pad.DrawString("1", new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, 27));

			pad.DrawString(selectedVertex.ConnectedComponent.MinDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(27, h - 27));
			pad.DrawString(selectedVertex.ConnectedComponent.MaxDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(w - 42, h - 27));

			for (int i = 0; i < numBins; i++) {
				pad.DrawLine(blackPen, (int)((i + 1)*((w-60)/numBins)) + 30, h - 33, (int)((i + 1)*((w-60)/numBins)) + 30, h - 27);

				//
				// Now draw this far from the left of the graphic:
				//	(int)((i+0.5)*((w-60)/numBins)) + 30
				// Look for values starting at:
				//	minDegree + i*((maxDegree - minDegree) / numBins)
				int numInThisBin = 0;
				double minBin = minDegree + ((double)i)*((maxDegree - minDegree) / (double)numBins);
				double maxBin = minDegree + ((double)(i + 1))*((maxDegree - minDegree) / (double)numBins);

				foreach(DictionaryEntry de in selectedVertex.ConnectedComponent.Vertices) {
					Vertex v = (Vertex)(de.Value);
					
					if ((((double)v.ConnectedTo.Count) >= minBin) &&
					    (((double)v.ConnectedTo.Count) < maxBin)) 
					{
						numInThisBin++;
					}
				}

				Trace.Write("drawDegreeDistribution", "binMin = " + minBin.ToString() + "; binMax = " + maxBin.ToString() + "; numinBucket = " + numInThisBin.ToString() + "; numTotal = " + selectedVertex.ConnectedComponent.Vertices.Count.ToString());

				float pK = ((float)numInThisBin) / ((float)selectedVertex.ConnectedComponent.Vertices.Count);
				pad.FillEllipse(blackBrush, (int)((i+0.5)*((w-60)/numBins)) + 30 - 4, 30 + (int)((1 - pK)*(h - 60)) - 4, 8, 8);
			}

			Random r = new Random();
			picName = "dd" + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			graphDegreeDistribution.ImageUrl = "graphs/" + picName;
		}


		/// <summary>
		/// Draw the degree k of each node vs. its average path length for the currently active connected component
		/// </summary>
		private void drawAveragePathLength() 
		{
			Vertex selectedVertex = State.SelectedVertex;
			int w = Int32.Parse(graphPathLength.Width.Value.ToString());
			int h = Int32.Parse(graphPathLength.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("k", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF((w / 2) - 3, h - 27));
			pad.DrawString("A", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(10, (h / 2) - 28));
			pad.DrawString("P", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(10, (h / 2) - 13));
			pad.DrawString("L", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(10, (h / 2) + 2));
			pad.DrawString("(k)", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(6, (h / 2) + 17));

			pad.DrawString(((int)selectedVertex.ConnectedComponent.MinAveragePathLength).ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, h - 40));
			pad.DrawString(((int)selectedVertex.ConnectedComponent.MaxAveragePathLength).ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, 27));

			pad.DrawString(selectedVertex.ConnectedComponent.MinDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(27, h - 27));
			pad.DrawString(selectedVertex.ConnectedComponent.MaxDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(w - 42, h - 27));
			
			foreach(DictionaryEntry de in selectedVertex.ConnectedComponent.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				pad.DrawEllipse(grayPen, ((int)v.xAverageDistance), ((int)v.yAverageDistance), 4, 4);
			}

			// selected vertex!
			pad.DrawEllipse(redPen, ((int)selectedVertex.xAverageDistance), ((int)selectedVertex.yAverageDistance), 4, 4);

			Random r = new Random();
			picName = "apl" + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			graphPathLength.ImageUrl = "graphs/" + picName;
		}


		/// <summary>
		/// Processes a click on the path length graph
		/// </summary>
		/// <param name="sender">The path length graph</param>
		/// <param name="e">Gives position of click to know which vertex to change to</param>
		protected void PathLength_Clicked(object sender, ImageClickEventArgs e) 
		{
			Vertex selected = State.SelectedVertex;
			double dist;

			Trace.Write("PathLength_Clicked", "Beginning...");

			Trace.Write("PathLength_Clicked", "e.X = " + e.X.ToString() + "; e.Y = " + e.Y.ToString());
		
			foreach(DictionaryEntry de in selected.ConnectedComponent.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selected) continue; // cycle through circles on that spot

				Trace.Write("PathLength_Clicked", "v.xAverageDistance = " + v.xAverageDistance.ToString() 
					+ "; v.yAverageDistance = " + v.yAverageDistance.ToString());

				dist = Math.Sqrt(((v.xAverageDistance + 2) - e.X)*((v.xAverageDistance + 2) - e.X) 
					+ ((v.yAverageDistance + 2) - e.Y)*((v.yAverageDistance + 2) - e.Y));

				if (dist <= 2) 
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					Trace.Write("PathLength_Clicked", "Ending...");

					return;
				}
			}

			Trace.Write("PathLength_Clicked", "Ending...");
		}

		/// <summary>
		/// Calls drawSpectralGraph(...) for each of the three matrix types.
		/// </summary>
		private void drawSpectralDensity()
		{
			drawSpectralGraph(EigenType.Adjacency, adjSpec);
			drawSpectralGraph(EigenType.Laplacian, lapSpec);
			drawSpectralGraph(EigenType.NormalizedLaplacian, normLapSpec);
		}

		/// <summary>
		/// Draw the spectral density graph
		/// </summary>
		private void drawSpectralGraph(EigenType eigenType, ImageButton ib) 
		{
			Vertex selectedVertex = State.SelectedVertex;
			ConnectedComponent cc = selectedVertex.ConnectedComponent;

			int w = Int32.Parse(ib.Width.Value.ToString());
			int h = Int32.Parse(ib.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("\u03BB/\u221A(Np(1-p))", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF((w / 2) - 3, h - 27));
			pad.DrawString("\u03C1\u221A", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(7, (h / 2) - 23));
			
			ArrayList eigens = new ArrayList(cc.NumVertices);
			for (int i = 0; i < cc.NumVertices; i++)
			{
				switch (eigenType) 
				{
					case EigenType.Adjacency:
						eigens.Add(cc.adjEigens[i]);
						break;
					case EigenType.Laplacian:
						eigens.Add(cc.lapEigens[i]);
						break;
					case EigenType.NormalizedLaplacian:
						eigens.Add(cc.normLapEigens[i]);
						break;
				}
			}

			// !! No need to sort here
			//eigens.Sort();

			//
			// Calculate UNSCALED coordinates of spectral density graph
			//
			double[] unsX = new double[eigens.Count];
			double[] unsY = new double[eigens.Count];

			double minX = double.MaxValue;
			double maxX = double.MinValue;
			
			double minY = double.MaxValue;
			double maxY = double.MinValue;

			for (int i = 0; i < eigens.Count; i++)
			{
				double p, result, eigen, root;
				int err;

				eigen = (double)eigens[i];
		
				p = 0.0;
				for (int j = 0; j < eigens.Count; j++)
				{
					Dirac.delta(10.0, (eigen - ((double)eigens[j])), 0.000000001, out result, out err);
					
					if (err != 0)
					{
						throw new Exception("drawSpectralGraph: Dirac.delta() returned error code " + err.ToString());
					}
					
					if (Math.Abs(result) > 0.0000000001)
					{
						p += (1.0 / eigens.Count) * result;
					}
				}

				if ((((double)eigens.Count) * p * (1.0 - p)) != Math.Abs(((double)eigens.Count) * p * (1.0 - p)))
				{
					pad.DrawString("Warning: absolute value of radicand required", new Font(FontFamily.GenericSansSerif, 7, FontStyle.Regular), blackBrush, new PointF(35, 35));
				}

				root = Math.Sqrt(Math.Abs(((double)eigens.Count) * p * (1.0 - p)));
				unsX[i] = eigen / root;
				unsY[i] = p * root;

				if (unsX[i] < minX)
				{
					minX = unsX[i];
				}
				if (unsX[i] > maxX)
				{
					maxX = unsX[i];
				}
				if (unsY[i] < minY)
				{
					minY = unsY[i];
				}
				if (unsY[i] > maxY)
				{
					maxY = unsY[i];
				}
			}

			//
			// Scale points and draw
			//
			double x, y;
			for (int i = 0; i < eigens.Count; i++)
			{
				x = 32.0 + (1.0 - (((double)(maxX - unsX[i])) 
					/ (maxX - minX))) * ((double)(w - 60));
				y = 28.0 + (((double)(maxY - unsY[i])) 
					/ ((double)(maxY - minY))) * ((double)(h - 60));

				pad.DrawEllipse(grayPen, (int)x, (int)y, 4, 4);
			}

			pad.DrawString(minY.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(7, h - 50));
			pad.DrawString(maxY.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(7, 17));

			pad.DrawString(minX.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, h - 27));
			pad.DrawString(maxY.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(w - 52, h - 27));

			Random r = new Random();
			picName = "specden" + eigenType.ToString() + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			ib.ImageUrl = "graphs/" + picName;
		}


		/// <summary>
		/// Draw the cluster coefficient of the currently active connected component
		/// </summary>
		private void drawClusterCoefficient() 
		{
			Vertex selectedVertex = State.SelectedVertex;
			int w = Int32.Parse(graphClusterCoefficient.Width.Value.ToString());
			int h = Int32.Parse(graphClusterCoefficient.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("k", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF((w / 2) - 3, h - 27));
			pad.DrawString("C(k)", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(1, (h / 2) - 5));

			pad.DrawString("0", new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, h - 40));
			pad.DrawString("1", new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(17, 27));

			pad.DrawString(selectedVertex.ConnectedComponent.MinDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(27, h - 27));
			pad.DrawString(selectedVertex.ConnectedComponent.MaxDegree.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(w - 42, h - 27));
			
			foreach(DictionaryEntry de in selectedVertex.ConnectedComponent.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (Double.IsNaN(v.ClusterCoeff) == false) 
				{
					pad.DrawEllipse(grayPen, ((int)v.xCC), ((int)v.yCC), 4, 4);
				}
			}

			// selected vertex!
			if (Double.IsNaN(selectedVertex.ClusterCoeff) == false) 
			{
				pad.DrawEllipse(redPen, ((int)selectedVertex.xCC), ((int)selectedVertex.yCC), 4, 4);
			}

			Random r = new Random();
			picName = "cc" + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			graphClusterCoefficient.ImageUrl = "graphs/" + picName;
		}


		/// <summary>
		/// Processes a click on the cluster coefficient graph
		/// </summary>
		/// <param name="sender">The cluster coefficient graph imagebutton</param>
		/// <param name="e">Gives position of click to know which vertex to change to</param>
		protected void ClusterCoefficient_Clicked(object sender, ImageClickEventArgs e) 
		{
			Vertex selected = State.SelectedVertex;
			double dist;

			Trace.Write("ClusterCoefficient_Clicked", "Beginning...");

			Trace.Write("ClusterCoefficient_Clicked", "e.X = " + e.X.ToString() + "; e.Y = " + e.Y.ToString());
		
			foreach(DictionaryEntry de in selected.ConnectedComponent.Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selected) continue; // cycle through circles on that spot

				Trace.Write("ClusterCoefficient_Clicked", "v.xCC = " + v.xCC.ToString() + "; v.yCC = " + v.yCC.ToString());

				dist = Math.Sqrt(((v.xCC + 2) - e.X)*((v.xCC + 2) - e.X) + ((v.yCC + 2) - e.Y)*((v.yCC + 2) - e.Y));

				if (dist <= 2) 
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					Trace.Write("ClusterCoefficient_Clicked", "Ending...");

					return;
				}
			}

			Trace.Write("ClusterCoefficient_Clicked", "Ending...");
		}


		/// <summary>
		/// Draw the eigenvalues of the currently active connected component.  This function draws eigenvalues 
		/// for each of the three eigenvalue types.
		/// </summary>
		/// <param name="ib">ImageButton whose picture to draw</param>
		/// <param name="eigenType">Adjacency, Laplacian, or NormalizedLaplacian</param>
		private void drawEigenvalues(ImageButton ib, EigenType eigenType) 
		{
			Trace.Write("drawEigenvalues", "Entering...");

			Vertex selectedVertex = State.SelectedVertex;
			ConnectedComponent cc = selectedVertex.ConnectedComponent;

			int w = Int32.Parse(ib.Width.Value.ToString());
			int h = Int32.Parse(ib.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			ArrayList eigens = new ArrayList(cc.NumVertices);
			for (int i = 0; i < cc.NumVertices; i++)
			{
				switch (eigenType) 
				{
					case EigenType.Adjacency:
						eigens.Add(cc.adjEigens[i]);
						break;
					case EigenType.Laplacian:
						eigens.Add(cc.lapEigens[i]);
						break;
					case EigenType.NormalizedLaplacian:
						eigens.Add(cc.normLapEigens[i]);
						break;
				}
			}
			eigens.Sort();

			double minEigen = (double)(eigens[0]);
			double maxEigen = (double)(eigens[eigens.Count - 1]);

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("Eigenvalue", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, 
				new PointF((w / 2) - 30, h - 27));
			pad.DrawString("\u03BB", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, 
				new PointF(5, (h / 2) - 5));

			pad.DrawString(((int)minEigen).ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(26 - (((int)minEigen).ToString().Length * 8), h - 38));
			pad.DrawString(((int)maxEigen).ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(26 - (((int)maxEigen).ToString().Length * 8), 27));

			pad.DrawString("1", new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(27, h - 27));
			pad.DrawString(eigens.Count.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(w - 42, h - 27));

			//
			// Now draw each eigenvalue...
			//
			for (int i = 0; i < eigens.Count; i++)
			{
				double x, y;
				x = 32.0 + (1.0 - (((double)(eigens.Count - i)) / ((double)(eigens.Count)))) * ((double)(w - 60));
				y = 30.0 + (((double)(maxEigen - ((double)eigens[i]))) / ((double)(maxEigen - minEigen))) * ((double)(h - 60)) - 2;
				pad.DrawEllipse(grayPen, (int)x, (int)y, 4, 4);
			}

			Random r = new Random();
			picName = "eigval" + eigenType.ToString() + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			ib.ImageUrl = "graphs/" + picName;

			Trace.Write("drawEigenvalues", "Exiting...");
		}


		/// <summary>
		/// Draw the inverse participation ratio for the currently active connected component.  This function draws
		/// the IVP for each of the three eigenvalue types.
		/// </summary>
		/// <param name="ib">ImageButton whose picture to draw</param>
		/// <param name="eigenType">Adjacency, Laplacian, or NormalizedLaplacian</param>
		private void drawIVP(ImageButton ib, EigenType eigenType) 
		{
			Trace.Write("drawIVP", "Entering...");

			Vertex selectedVertex = State.SelectedVertex;
			ConnectedComponent cc = selectedVertex.ConnectedComponent;

			int w = Int32.Parse(ib.Width.Value.ToString());
			int h = Int32.Parse(ib.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			ArrayList ivp = new ArrayList(cc.NumVertices);
			for (int i = 0; i < cc.NumVertices; i++)
			{
				switch (eigenType) 
				{
					case EigenType.Adjacency:
						ivp.Add(cc.adjIVP[i]);
						break;
					case EigenType.Laplacian:
						ivp.Add(cc.lapIVP[i]);
						break;
					case EigenType.NormalizedLaplacian:
						ivp.Add(cc.normLapIVP[i]);
						break;
				}
			}
			ivp.Sort();

			double minVal = (double)(ivp[0]);
			double maxVal = (double)(ivp[ivp.Count - 1]);

			double minEigen = 0;
			double maxEigen = 0;
			switch (eigenType)
			{
				case EigenType.Adjacency:
					minEigen = cc.adjEigens[0];
					maxEigen = cc.adjEigens[cc.NumVertices - 1];
					break;
				case EigenType.Laplacian:
					minEigen = cc.lapEigens[0];
					maxEigen = cc.lapEigens[cc.NumVertices - 1];
					break;
				case EigenType.NormalizedLaplacian:
					minEigen = cc.normLapEigens[0];
					maxEigen = cc.normLapEigens[cc.NumVertices - 1];
					break;
			}

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);

			pad.DrawString("Eigenvalue", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, 
				new PointF((w / 2) - 30, h - 27));

			pad.DrawString(minEigen.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(20, h - 27));
			pad.DrawString(maxEigen.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), 
				blackBrush, new PointF(w - 49, h - 27));

			pad.DrawString(minVal.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(7, h - 50));
			pad.DrawString(maxVal.ToString(), new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(7, 17));

			//
			// Now draw each eigenvalue...
			//
			for (int i = 0; i < cc.NumVertices; i++)
			{
				double x = 0.0;
				double y = 0.0;

				if (minEigen == maxEigen)
				{
					x = 30.0 + .5 * ((double)w - 60.0);
				}
				else
				{
					switch (eigenType) 
					{
						case EigenType.Adjacency:
							x = 32.0 + (1.0 - (((double)(maxEigen - cc.adjEigens[i])) /
								((double)(maxEigen - minEigen)))) * ((double)(w - 60));
							break;
						case EigenType.Laplacian:
							x = 32.0 + (1.0 - (((double)(maxEigen - cc.lapEigens[i])) /
								((double)(maxEigen - minEigen)))) * ((double)(w - 60));
							break;
						case EigenType.NormalizedLaplacian:
							x = 32.0 + (1.0 - (((double)(maxEigen - cc.normLapEigens[i])) /
								((double)(maxEigen - minEigen)))) * ((double)(w - 60));
							break;
					}
				}
					
				if (minVal == maxVal)
				{
					y = 30.0 + .5 * ((double)h - 60) - 2.0;
				}
				else
				{
					switch (eigenType) 
					{
						case EigenType.Adjacency:
							y = 28.0 + (1.0 - (((double)(maxVal - cc.adjIVP[i])) /
								((double)(maxVal - minVal)))) * ((double)(h - 60));
							break;
						case EigenType.Laplacian:
							y = 28.0 + (1.0 - (((double)(maxVal - cc.lapIVP[i])) /
								((double)(maxVal - minVal)))) * ((double)(h - 60));
							break;
						case EigenType.NormalizedLaplacian:
							y = 28.0 + (1.0 - (((double)(maxVal - cc.normLapIVP[i])) /
								((double)(maxVal - minVal)))) * ((double)(h - 60));
							break;
					}
				}

				pad.DrawEllipse(grayPen, (int)x, (int)y, 4, 4);
			}

			Random r = new Random();
			picName = "ivp" + eigenType.ToString() + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			ib.ImageUrl = "graphs/" + picName;

			Trace.Write("drawIVP", "Exiting...");
		}


		/// <summary>
		/// Sorting class for drawing graphs sorted by different means.  This class sorts by eigenvector value.
		/// </summary>
		public class sortByEigenvector : IComparer  
		{
			int _which;
			EigenType _type;

			public sortByEigenvector(EigenType type, int which)
			{
				_which = which;
				_type = type;
			}

			int IComparer.Compare(Object x, Object y)
			{
				double xval = 0.0;
				double yval = 0.0;

				switch (_type)
				{
					case EigenType.Adjacency:
						xval = ((Vertex)x).getAdjacencyEigenvalue(_which);
						yval = ((Vertex)y).getAdjacencyEigenvalue(_which);
						break;
					case EigenType.Laplacian:
						xval = ((Vertex)x).getLaplacianEigenvalue(_which);
						yval = ((Vertex)y).getLaplacianEigenvalue(_which);
						break;
					case EigenType.NormalizedLaplacian:
						xval = ((Vertex)x).getNormalizedLaplacianEigenvalue(_which);
						yval = ((Vertex)y).getNormalizedLaplacianEigenvalue(_which);
						break;
				}

				if (xval < yval)
				{ 
					return -1;
				} 
				else if (xval == yval)
				{
					return 0;
				}
				else
				{
					return 1;
				}
			}
		}


		/// <summary>
		/// Sorting class for drawing graphs sorted by different means.  This class sorts by node degree.
		/// </summary>
		public class sortByDegree : IComparer  
		{
			int IComparer.Compare(Object x, Object y)
			{
				double xval = ((Vertex)x).ConnectedTo.Count;
				double yval = ((Vertex)y).ConnectedTo.Count;

				if (xval < yval)
				{ 
					return -1;
				} 
				else if (xval == yval)
				{
					return 0;
				}
				else
				{
					return 1;
				}
			}
		}


		/// <summary>
		/// Draws the eigenvector graphs for the currently active connected component. 
		/// </summary>
		/// <param name="ib">The imagebutton to write to</param>
		/// <param name="eigenType">Adjacency, Laplacian, or NormalizedLaplacian.  With which, specifies which of the 9 graphs we're drawing.</param>
		/// <param name="which">Which eigenvector to draw.  With eigenType, specifies which of the 9 graphs we're drawing.</param>
		private void drawEigenvectors(ImageButton ib, EigenType eigenType, int which) 
		{
			Trace.Write("drawEigenvectors", "Entering...");

			Vertex selectedVertex = State.SelectedVertex;
			ConnectedComponent cc = selectedVertex.ConnectedComponent;

			int w = Int32.Parse(ib.Width.Value.ToString());
			int h = Int32.Parse(ib.Height.Value.ToString());
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			//
			// Get the sort order
			//

			bool sortByValue;
			if (ib.Attributes["sort"] == "degree")
			{
				sortByValue = false;
			}
			else
			{
				sortByValue = true;
			}


			//
			// Get the min/max value for displaying on the axes
			//
			// Also:
			// Sorts vertices according to preferece -- uses sortedlist with (key,value) = (degree, vertex) or (eigvec, vertex)
			//

			ArrayList vertices = new ArrayList(cc.NumVertices);
			foreach (DictionaryEntry de in cc.Vertices)
			{
				Vertex v = (Vertex)(de.Value);

				vertices.Add(v);
			}

			IComparer myComparer;
			myComparer = new sortByEigenvector(eigenType, which);
			vertices.Sort(myComparer);
			double minE = 0.0; 
			double maxE = 0.0; 

			switch (eigenType)
			{
				case EigenType.Adjacency:
					minE = ((Vertex)vertices[0]).getAdjacencyEigenvalue(which);
					maxE = ((Vertex)vertices[vertices.Count - 1]).getAdjacencyEigenvalue(which);
					break;
				case EigenType.Laplacian:
					minE = ((Vertex)vertices[0]).getLaplacianEigenvalue(which);
					maxE = ((Vertex)vertices[vertices.Count - 1]).getLaplacianEigenvalue(which);
					break;
				case EigenType.NormalizedLaplacian:
					minE = ((Vertex)vertices[0]).getNormalizedLaplacianEigenvalue(which);
					maxE = ((Vertex)vertices[vertices.Count - 1]).getNormalizedLaplacianEigenvalue(which);
					break;
			}

			if (sortByValue)
			{
				myComparer = new sortByEigenvector(eigenType, which);
			}
			else // sort by degree
			{
				myComparer = new sortByDegree();
			}

			// Now do the sort
			vertices.Sort(myComparer);


			//
			// Start drawing
			//

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			pad.DrawLine(bigBlackPen, 30, h - 30, w - 30, h - 30);
			pad.DrawLine(bigBlackPen, 30, h - 30, 30, 30);
	
			
			pad.DrawString("\u03BB", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF(3, (h / 2) - 5));

			//
			// Left-side max/min labels
			//
			switch (eigenType)
			{
				case EigenType.Adjacency:
					pad.DrawString(minE.ToString().Substring(0, Math.Min(minE.ToString().Length, 4)), 
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, h - 38));
					pad.DrawString(maxE.ToString().Substring(0, Math.Min(maxE.ToString().Length, 4)), 
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, 27));
					break;
				case EigenType.Laplacian:
					pad.DrawString(minE.ToString().Substring(0, Math.Min(minE.ToString().Length, 4)), 
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, h - 38));
					pad.DrawString(maxE.ToString().Substring(0, Math.Min(maxE.ToString().Length, 4)), 
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, 27));
					break;
				case EigenType.NormalizedLaplacian:
					pad.DrawString(minE.ToString().Substring(0, Math.Min(minE.ToString().Length, 4)), 
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, h - 38));
					pad.DrawString(maxE.ToString().Substring(0, Math.Min(maxE.ToString().Length, 4)),
						new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(3, 27));
					break;
			}

			//
			// Bottom labels
			//
			if (sortByValue)
			{
				pad.DrawString("Node (sorted by \u03BB)", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, new PointF((w / 2) - 30, h - 27));

				//We're not displaying here, since there is no scale
			}
			else
			{
				pad.DrawString("Node (sorted by degree)", new Font(FontFamily.GenericSansSerif, 9, FontStyle.Bold), blackBrush, 
					new PointF((w / 2) - 30, h - 27));

				pad.DrawString(((Vertex)vertices[0]).ConnectedTo.Count.ToString(), 
					new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(27, h - 27));
				pad.DrawString(((Vertex)vertices[vertices.Count - 1]).ConnectedTo.Count.ToString(), 
					new Font(FontFamily.GenericSansSerif, 8, FontStyle.Regular), blackBrush, new PointF(w - 42, h - 27));
			}

			//
			// Calculate x-value for eigenvector point
			//
			for (int i = 0; i < vertices.Count; i++)
			{
				Vertex v = (Vertex)vertices[i];

				if (sortByValue)
				{
					//
					// Sort by eigenvalue EQUALLY SPACES the results
					//
					switch (eigenType)
					{
						case EigenType.Adjacency:
							v.AdjacencyEigenInformation.x[which] = 32.0 + (1.0 - (((double)(vertices.Count - i)) / 
								((double)(vertices.Count)))) * ((double)(w - 60));
							break;
						case EigenType.Laplacian:
							v.LaplacianEigenInformation.x[which] = 32.0 + (1.0 - (((double)(vertices.Count - i)) / 
								((double)(vertices.Count)))) * ((double)(w - 60));
							break;
						case EigenType.NormalizedLaplacian:
							v.NormalizedLaplacianEigenInformation.x[which] = 32.0 + (1.0 - (((double)(vertices.Count - i)) / 
								((double)(vertices.Count)))) * ((double)(w - 60));
							break;
					}
				}
				else // SORTED BY NODE DEGREE
				{
					//
					// Set min/max values (easier than the value case)
					//
					double minDegree = ((Vertex)vertices[0]).ConnectedTo.Count;
					double maxDegree = ((Vertex)vertices[vertices.Count - 1]).ConnectedTo.Count;

					//
					// Set x-value
					//
					switch (eigenType)
					{
						case EigenType.Adjacency:
							v.AdjacencyEigenInformation.x[which] = 32.0 + 
								(1.0 - (((double)(maxDegree - v.ConnectedTo.Count)) 
								/ ((double)(maxDegree - minDegree)))) * ((double)(w - 60));
							break;
						case EigenType.Laplacian:
							v.LaplacianEigenInformation.x[which] = 32.0 + 
								(1.0 - (((double)(maxDegree - v.ConnectedTo.Count)) 
								/ ((double)(maxDegree - minDegree)))) * ((double)(w - 60));
							break;
						case EigenType.NormalizedLaplacian:
							v.NormalizedLaplacianEigenInformation.x[which] = 32.0 + 
								(1.0 - (((double)(maxDegree - v.ConnectedTo.Count)) 
								/ ((double)(maxDegree - minDegree)))) * ((double)(w - 60));
							break;
					}
				}
			}

			//
			// Draw points
			//
			for (int i = 0; i < vertices.Count; i++)
			{
				Vertex v = (Vertex)vertices[i];

				if (v == selectedVertex) continue;

				switch (eigenType)
				{
					case EigenType.Adjacency:
						pad.DrawEllipse(grayPen, ((int)v.AdjacencyEigenInformation.x[which]), 
							((int)v.AdjacencyEigenInformation.y[which]), 4, 4);
						break;
					case EigenType.Laplacian:
						pad.DrawEllipse(grayPen, ((int)v.LaplacianEigenInformation.x[which]), 
							((int)v.LaplacianEigenInformation.y[which]), 4, 4);
						break;
					case EigenType.NormalizedLaplacian:
						pad.DrawEllipse(grayPen, ((int)v.NormalizedLaplacianEigenInformation.x[which]), 
							((int)v.NormalizedLaplacianEigenInformation.y[which]), 4, 4);
						break;
				}
			}

			//
			// Draw selectedVertex
			//
			switch (eigenType)
			{
				case EigenType.Adjacency:
					pad.FillEllipse(redBrush, ((int)selectedVertex.AdjacencyEigenInformation.x[which])-1, 
						((int)selectedVertex.AdjacencyEigenInformation.y[which])-1, 6, 6);
					break;
				case EigenType.Laplacian:
					pad.FillEllipse(redBrush, ((int)selectedVertex.LaplacianEigenInformation.x[which])-1, 
						((int)selectedVertex.LaplacianEigenInformation.y[which])-1, 6, 6);
					break;
				case EigenType.NormalizedLaplacian:
					pad.FillEllipse(redBrush, ((int)selectedVertex.NormalizedLaplacianEigenInformation.x[which])-1, 
						((int)selectedVertex.NormalizedLaplacianEigenInformation.y[which])-1, 6, 6);
					break;
			}

			Random r = new Random();
			picName = "eigvec" + eigenType.ToString() + which.ToString() + r.Next() + ".gif";
			pic.Save(Server.MapPath("graphs/" + picName), ImageFormat.Gif);
			State.Graph.addFile(Server.MapPath("graphs/" + picName));
			ib.ImageUrl = "graphs/" + picName;

			Trace.Write("drawEigenvectors", "Exiting...");
		}


		/// <summary>
		/// Process a change in desired sort order
		/// </summary>
		/// <param name="sender">The radio button to be clicked</param>
		/// <param name="e">Unused</param>
		protected void Sort_Changed(object sender, System.EventArgs e)
		{
			switch (((RadioButton)sender).GroupName)
			{
				case "adjE0":
					if (adjE0Vec.Checked == true) adjacencyEigenvectorZero.Attributes["sort"] = "value";
					else adjacencyEigenvectorZero.Attributes["sort"] = "degree";
					drawEigenvectors(adjacencyEigenvectorZero, EigenType.Adjacency, 0);
					break;
				case "adjE1":
					if (adjE1Vec.Checked == true) adjacencyEigenvectorOne.Attributes["sort"] = "value";
					else adjacencyEigenvectorOne.Attributes["sort"] = "degree";
					drawEigenvectors(adjacencyEigenvectorOne, EigenType.Adjacency, 1);
					break;
				case "adjE2":
					if (adjE2Vec.Checked == true) adjacencyEigenvectorTwo.Attributes["sort"] = "value";
					else adjacencyEigenvectorTwo.Attributes["sort"] = "degree";
					drawEigenvectors(adjacencyEigenvectorTwo, EigenType.Adjacency, 2);
					break;
				case "adjE3":
					if (adjE3Vec.Checked == true) adjacencyEigenvectorThree.Attributes["sort"] = "value";
					else adjacencyEigenvectorThree.Attributes["sort"] = "degree";
					drawEigenvectors(adjacencyEigenvectorThree, EigenType.Adjacency, 3);
					break;

				case "lapE0":
					if (lapE0Vec.Checked == true) laplacianEigenvectorZero.Attributes["sort"] = "value";
					else laplacianEigenvectorZero.Attributes["sort"] = "degree";
					drawEigenvectors(laplacianEigenvectorZero, EigenType.Laplacian, 0);
					break;
				case "lapE1":
					if (lapE1Vec.Checked == true) laplacianEigenvectorOne.Attributes["sort"] = "value";
					else laplacianEigenvectorOne.Attributes["sort"] = "degree";
					drawEigenvectors(laplacianEigenvectorOne, EigenType.Laplacian, 1);
					break;
				case "lapE2":
					if (lapE2Vec.Checked == true) laplacianEigenvectorTwo.Attributes["sort"] = "value";
					else laplacianEigenvectorTwo.Attributes["sort"] = "degree";
					drawEigenvectors(laplacianEigenvectorTwo, EigenType.Laplacian, 2);
					break;
				case "lapE3":
					if (lapE3Vec.Checked == true) laplacianEigenvectorThree.Attributes["sort"] = "value";
					else laplacianEigenvectorThree.Attributes["sort"] = "degree";
					drawEigenvectors(laplacianEigenvectorThree, EigenType.Laplacian, 3);
					break;

				case "normLapE0":
					if (normLapE0Vec.Checked == true) normLaplacianEigenvectorZero.Attributes["sort"] = "value";
					else normLaplacianEigenvectorZero.Attributes["sort"] = "degree";
					drawEigenvectors(normLaplacianEigenvectorZero, EigenType.NormalizedLaplacian, 0);
					break;
				case "normLapE1":
					if (normLapE1Vec.Checked == true) normLaplacianEigenvectorOne.Attributes["sort"] = "value";
					else normLaplacianEigenvectorOne.Attributes["sort"] = "degree";
					drawEigenvectors(normLaplacianEigenvectorOne, EigenType.NormalizedLaplacian, 1);
					break;
				case "normLapE2":
					if (normLapE2Vec.Checked == true) normLaplacianEigenvectorTwo.Attributes["sort"] = "value";
					else normLaplacianEigenvectorTwo.Attributes["sort"] = "degree";
					drawEigenvectors(normLaplacianEigenvectorTwo, EigenType.NormalizedLaplacian, 2);
					break;
				case "normLapE3":
					if (normLapE3Vec.Checked == true) normLaplacianEigenvectorThree.Attributes["sort"] = "value";
					else normLaplacianEigenvectorThree.Attributes["sort"] = "degree";
					drawEigenvectors(normLaplacianEigenvectorThree, EigenType.NormalizedLaplacian, 3);
					break;
			}
		}


		/// <summary>
		/// Redraws every eigengraph (i.e. if a new vertex is selected)
		/// </summary>
		private void drawEigenInformation()
		{
			// First draw eigenvalue spectra
			drawEigenvalues(adjacencyEigenvalues, EigenType.Adjacency);
			drawEigenvalues(laplacianEigenvalues, EigenType.Laplacian);
			drawEigenvalues(normLaplacianEigenvalues, EigenType.NormalizedLaplacian);

			// Now draw eigenvectors (all 9 of them)
			drawEigenvectors(adjacencyEigenvectorZero, EigenType.Adjacency, 0);
			drawEigenvectors(adjacencyEigenvectorOne, EigenType.Adjacency, 1);
			drawEigenvectors(adjacencyEigenvectorTwo, EigenType.Adjacency, 2);
			drawEigenvectors(adjacencyEigenvectorThree, EigenType.Adjacency, 3);

			drawEigenvectors(laplacianEigenvectorZero, EigenType.Laplacian, 0);
			drawEigenvectors(laplacianEigenvectorOne, EigenType.Laplacian, 1);
			drawEigenvectors(laplacianEigenvectorTwo, EigenType.Laplacian, 2);
			drawEigenvectors(laplacianEigenvectorThree, EigenType.Laplacian, 3);

			drawEigenvectors(normLaplacianEigenvectorZero, EigenType.NormalizedLaplacian, 0);
			drawEigenvectors(normLaplacianEigenvectorOne, EigenType.NormalizedLaplacian, 1);
			drawEigenvectors(normLaplacianEigenvectorTwo, EigenType.NormalizedLaplacian, 2);
			drawEigenvectors(normLaplacianEigenvectorThree, EigenType.NormalizedLaplacian, 3);

			// Draw IVP
			drawIVP(adjIVP, EigenType.Adjacency);
			drawIVP(lapIVP, EigenType.Laplacian);
			drawIVP(normLapIVP, EigenType.NormalizedLaplacian);

		}


		/// <summary>
		/// Processes a click on one of the eigenvector graphs.
		/// </summary>
		/// <param name="sender">Used to determine which eigenvector graph was clicked.</param>
		/// <param name="e">Gives position of click to know which vertex to change to</param>
		protected void eigenGraph_Clicked(object sender, ImageClickEventArgs e) 
		{
			// Do some stuff to figure out which vertex was clicked, and run
			// vertex_changed(...)
			
			EigenType type = EigenType.Adjacency;
			int which = 0;
			Vertex selectedVertex = State.SelectedVertex;

			//
			// Determine which graph was clicked (this is better than having 9 _Clicked functions)
			//
			if (((ImageButton)sender) == adjacencyEigenvectorZero)
			{
				type = EigenType.Adjacency;
				which = 0;
			}
			else if (((ImageButton)sender) == adjacencyEigenvectorOne)
			{
				type = EigenType.Adjacency;
				which = 1;
			}
			else if (((ImageButton)sender) == adjacencyEigenvectorTwo)
			{
				type = EigenType.Adjacency;
				which = 2;
			}
			else if (((ImageButton)sender) == adjacencyEigenvectorThree)
			{
				type = EigenType.Adjacency;
				which = 3;
			}
			else if (((ImageButton)sender) == laplacianEigenvectorZero)
			{
				type = EigenType.Laplacian;
				which = 0;
			}
			else if (((ImageButton)sender) == laplacianEigenvectorOne)
			{
				type = EigenType.Laplacian;
				which = 1;
			}
			else if (((ImageButton)sender) == laplacianEigenvectorTwo)
			{
				type = EigenType.Laplacian;
				which = 2;
			}
			else if (((ImageButton)sender) == laplacianEigenvectorThree)
			{
				type = EigenType.Laplacian;
				which = 3;
			}
			else if (((ImageButton)sender) == normLaplacianEigenvectorZero)
			{
				type = EigenType.NormalizedLaplacian;
				which = 0;
			}
			else if (((ImageButton)sender) == normLaplacianEigenvectorOne)
			{
				type = EigenType.NormalizedLaplacian;
				which = 1;
			}
			else if (((ImageButton)sender) == normLaplacianEigenvectorTwo)
			{
				type = EigenType.NormalizedLaplacian;
				which = 2;
			}
			else if (((ImageButton)sender) == normLaplacianEigenvectorThree)
			{
				type = EigenType.NormalizedLaplacian;
				which = 3;
			}
			else
			{
				throw new Exception("Bizarre imagebutton was clicked.");
			}

			//
			// Figure out which vertex was clicked
			//
			foreach(DictionaryEntry de in selectedVertex.ConnectedComponent.Vertices)
			{
				double dist = 0.0;
				Vertex v = (Vertex)de.Value;

				if (v == selectedVertex) continue;
				
				switch (type)
				{
					case EigenType.Adjacency:
						dist = Math.Sqrt((v.AdjacencyEigenInformation.x[which] + 0 - e.X)*(v.AdjacencyEigenInformation.x[which] + 0 - e.X)
							+ (v.AdjacencyEigenInformation.y[which] + 4 - e.Y)*(v.AdjacencyEigenInformation.y[which] + 4 - e.Y));
						break;
					case EigenType.Laplacian:
						dist = Math.Sqrt((v.LaplacianEigenInformation.x[which] + 0 - e.X)*(v.LaplacianEigenInformation.x[which] + 0 - e.X)
							+ (v.LaplacianEigenInformation.y[which] + 4 - e.Y)*(v.LaplacianEigenInformation.y[which] + 4 - e.Y));
						break;
					case EigenType.NormalizedLaplacian:
						dist = Math.Sqrt((v.NormalizedLaplacianEigenInformation.x[which] + 0 - e.X)*(v.NormalizedLaplacianEigenInformation.x[which] + 0 - e.X)
							+ (v.NormalizedLaplacianEigenInformation.y[which] + 4 - e.Y)*(v.NormalizedLaplacianEigenInformation.y[which] + 4 - e.Y));
						break;
				}

				if (dist <= 3)
				{
					State.SelectedVertex = v;

					changeVertexToCurrent();

					return;
				}
			}
		}

		/// <summary>
		/// Processes a change in the current vertex.  Only changes UI, doesn't actually display information.  
		/// See changeVertexDisplay().
		/// </summary>
		private void changeVertexToCurrent()
		{
			Trace.Write("changeVertexToCurrent", "Vertex Changed to: " + State.SelectedVertex.Label + " (" + State.SelectedVertex.ID + ")");

			foreach(ListItem li in verticesLB.Items)
			{
				if (Int32.Parse(li.Value) == State.SelectedVertex.ID)
				{
					li.Selected = true;
				}
				else
				{
					li.Selected = false;
				}
			}

			edgesLB.Items.Clear();

			foreach(Vertex v in State.SelectedVertex.ConnectedTo) {
				edgesLB.Items.Add(new ListItem(v.Label, v.ID.ToString()));
			}

			if (State.SelectedVertex.hasComponentInfo == true)
			{
				btnTextOutput.Enabled = true;

				if (State.Graph.DisplayType == displayType.Normal)
				{
					gNormal.Visible = false;
					gLaplacian.Visible = true;
				}
				else
				{
					gLaplacian.Visible = false;
					gNormal.Visible = true;
				}

				connInfo.Text = "Connected component has " + State.SelectedVertex.ConnectedComponent.NumVertices.ToString() + " nodes, with"
					+" an average diameter of " + State.SelectedVertex.ConnectedComponent.AverageDiameter.ToString() + ".";
			}
			else if (State.SelectedVertex.ConnectedComponent.NumVertices > 1) 
			{
				btnTextOutput.Enabled = false;
				State.Graph.DisplayType = displayType.Normal;
				gNormal.Visible = false;
				gLaplacian.Visible = false;
				normalPanel.Visible = true;
				laplacianPanel.Visible = false;
				connInfo.Text = "Connected component has " + State.SelectedVertex.ConnectedComponent.NumVertices.ToString() + " nodes.";
			} 
			else 
			{
				btnTextOutput.Enabled = false;
				State.Graph.DisplayType = displayType.Normal;
				gNormal.Visible = false;
				gLaplacian.Visible = false;
				normalPanel.Visible = true;
				laplacianPanel.Visible = false;
				connInfo.Text = "Connected component has " + State.SelectedVertex.ConnectedComponent.NumVertices.ToString() + " node.";
			}

			changeVertexDisplay(State.SelectedVertex.ID);
		}


		/// <summary>
		/// Changes displayed information to reflect the currently active vertex.
		/// </summary>
		/// <param name="id">The ID number of the node we're changing to.</param>
		private void changeVertexDisplay(int id) {
			Vertex v = (Vertex) State.Graph.Vertices[id];

			State.Graph.deleteFiles();

			drawGraph();

			vDegree.Text = "Degree: " + v.ConnectedTo.Count;

			if (v.hasComponentInfo == true) {
				pnlShowComponent.Visible = false;
				pnlVertexComponentInfo.Visible = true;
				pnlComponent.Visible = true;

				drawDegreeDistribution();
				drawClusterCoefficient();
				drawAveragePathLength();
				drawSpectralDensity();
				drawEigenInformation();

				v.ConnectedComponent.drawLaplacianGraph(v, Server.MapPath("graphs/"));
				laplacianDisplay.ImageUrl = v.ConnectedComponent.LaplacianGraphLocation;

				vCC.Text = "C(k): " + v.ClusterCoeff.ToString();
				vDist.Text = "Dist(min,max,avg) = (" + v.MinDist + "," + v.MaxDist + "," + v.AvgDist + ")";

				vAdjE0.Text = "Adjacency \u03BB0: " + v.getAdjacencyEigenvalue(0).ToString();
				vAdjE1.Text = "Adjacency \u03BB1: " + v.getAdjacencyEigenvalue(1).ToString();
				vAdjE2.Text = "Adjacency \u03BB2: " + v.getAdjacencyEigenvalue(2).ToString();
				vAdjE3.Text = "Adjacency \u03BB3: " + v.getAdjacencyEigenvalue(3).ToString();
				
				vLapE0.Text = "Laplacian \u03BB0: " + v.getLaplacianEigenvalue(0).ToString();
				vLapE1.Text = "Laplacian \u03BB1: " + v.getLaplacianEigenvalue(1).ToString();
				vLapE2.Text = "Laplacian \u03BB2: " + v.getLaplacianEigenvalue(2).ToString();
				vLapE3.Text = "Laplacian \u03BB3: " + v.getLaplacianEigenvalue(3).ToString();
				
				vNormLapE0.Text = "Norm. Laplacian \u03BB0: " + v.getNormalizedLaplacianEigenvalue(0).ToString();
				vNormLapE1.Text = "Norm. Laplacian \u03BB1: " + v.getNormalizedLaplacianEigenvalue(1).ToString();
				vNormLapE2.Text = "Norm. Laplacian \u03BB2: " + v.getNormalizedLaplacianEigenvalue(2).ToString();
				vNormLapE3.Text = "Norm. Laplacian \u03BB3: " + v.getNormalizedLaplacianEigenvalue(3).ToString();

				StringBuilder lae = new StringBuilder();
				StringBuilder lle = new StringBuilder();
				StringBuilder lnle = new StringBuilder();
				for (int i = 0; i < v.ConnectedComponent.NumVertices; i++) 
				{
					lae.Append("\u03BB" + i.ToString() + ": " + v.ConnectedComponent.adjEigens[i] + "<br>");
					lle.Append("\u03BB" + i.ToString() + ": " + v.ConnectedComponent.lapEigens[i] + "<br>");
					lnle.Append("\u03BB" + i.ToString() + ": " + v.ConnectedComponent.normLapEigens[i] + "<br>");
				}
				lblAdjEigens.Text = lae.ToString();
				lblLapEigens.Text = lle.ToString();
				lblNormLapEigens.Text = lnle.ToString();
			} else {
				pnlShowComponent.Visible = true;
				pnlVertexComponentInfo.Visible = false;
				pnlComponent.Visible = false;
			}

			//Response.Write("DONE: " + DateTime.Now.ToLocalTime().ToString() + "<BR>");

			vName.Text = v.Label;
		}

		/// <summary>
		/// Initializes
		/// </summary>
		/// <param name="sender">Unused</param>
		/// <param name="e">Unused</param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			//Response.Write("BEGINNING: " + DateTime.Now.ToLocalTime().ToString() + "<BR>");

			Trace.Write("Page_Load", "Beginning...");

			if (IsPostBack == false) {
				//
				// Clean out the graphs/ folder
				//
				try
				{
					foreach (string s in Directory.GetFiles(Server.MapPath("graphs/")))
					{
						// If the file is more than a day old
						if (File.GetCreationTime(s) < DateTime.Now.AddDays(-1))
						{
							File.Delete(s);
						}
					}

					foreach (string s in Directory.GetFiles(Server.MapPath("export/")))
					{
						// If the file is more than a day old
						if (File.GetCreationTime(s) < DateTime.Now.AddDays(-1))
						{
							File.Delete(s);
						}
					}
				}
				catch
				{
					// forget about it...
				}

				//
				// Perform initial GUI setup
				//
				lblNormLapIVP.Text = "Inverse Participation Ratio (\u03BB vs. log(ivp(\u03BB)))";
				lblLapIVP.Text = "Inverse Participation Ratio (\u03BB vs. log(ivp(\u03BB)))";
				lblAdjIVP.Text = "Inverse Participation Ratio (\u03BB vs. log(ivp(\u03BB)))";

				outAdjSpec.Text = "Spectral Density (will be two columns, one for \u03BB/Sqrt(Np(1-p)), one for p*Sqrt(Np(1-p))";
				outLapSpec.Text = "Spectral Density (will be two columns, one for \u03BB/Sqrt(Np(1-p)), one for p*Sqrt(Np(1-p))";
				outNormLapSpec.Text = "Spectral Density (will be two columns, one for \u03BB/Sqrt(Np(1-p)), one for p*Sqrt(Np(1-p))";

				importFile.Visible = true;
				analysisDisplay.Visible = false;
				pnlCollapse.Visible = false;
				pnlTextOutput.Visible = false;
				pnlExportLinks.Visible = false;

				Unclick_outNode(sender, e);
				Unclick_outAdjEigenvalueInfo(sender, e);
				Unclick_outLapEigenvalueInfo(sender, e);
				Unclick_outNormLapEigenvalueInfo(sender, e);

				for (int i = 1; i < 100; i++)
				{
					numSimilarities.Items.Add(new ListItem(i.ToString(), i.ToString()));
				}

				normalPanel.Visible = true;
				laplacianPanel.Visible = false;

				adjacencyEigenvectorZero.Attributes.Add("sort", "value");
				adjacencyEigenvectorOne.Attributes.Add("sort", "value");
				adjacencyEigenvectorTwo.Attributes.Add("sort", "value");
				adjacencyEigenvectorThree.Attributes.Add("sort", "value");

				laplacianEigenvectorZero.Attributes.Add("sort", "value");
				laplacianEigenvectorOne.Attributes.Add("sort", "value");
				laplacianEigenvectorTwo.Attributes.Add("sort", "value");
				laplacianEigenvectorThree.Attributes.Add("sort", "value");

				normLaplacianEigenvectorZero.Attributes.Add("sort", "value");
				normLaplacianEigenvectorOne.Attributes.Add("sort", "value");
				normLaplacianEigenvectorTwo.Attributes.Add("sort", "value");
				normLaplacianEigenvectorThree.Attributes.Add("sort", "value");
			}

			Trace.Write("Page_Load", "Ending...");
		}


		/// <summary>
		/// IDE code
		/// </summary>
		private void InitializeComponent()
		{

		}


		/// <summary>
		/// IDE code. Required to get Page_Load working.
		/// </summary>
		/// <param name="e">Unused</param>
		override protected void OnInit(EventArgs e)
		{
			this.Load += new System.EventHandler(this.Page_Load);
			base.OnInit(e);
		}
	}

	/// <summary>
	/// A node in a graph
	/// </summary>
	public class Vertex {
		/// <summary>
		/// This node's identification number.  These numbers may be arbitrary, as long as they are unique over the graph.
		/// </summary>
		int _id;
		
		/// <summary>
		/// The string representing this node. I.e. "4 H11".
		/// </summary>
		string _label;

		/// <summary>
		/// The number of edges this vertex is connected to.  Unused?
		/// </summary>
		int _numRefs = 0;

		/// <summary>
		/// Distance (in hops) from this vertex to the rest of the graph.
		/// </summary>
		double _minDist, _maxDist, _avgDist;

		/// <summary>
		/// Coordinates for avg. path length graph.
		/// </summary>
		double _xAvgDist, _yAvgDist;

		/// <summary>
		/// Represents whether this node is internally regarded as a chemical.  Used in pseudo-modal projection w/ regex.
		/// </summary>
		bool _isChemical;

		/// <summary>
		/// Used in making hierarchical graphs, this represents whether this node is a peripheral node.
		/// </summary>
		bool _isPeripheral;

		/// <summary>
		/// Used to mark during searching.
		/// </summary>
		public bool marked;

		/// <summary>
		/// Graph coordinates in main graph.
		/// </summary>
		double _xpos, _ypos;

		/// <summary>
		/// Used in running F-R algorithm.
		/// </summary>
		double _xdisp, _ydisp;

		/// <summary>
		/// Graph coordinates in Laplacian LLE graph.
		/// </summary>
		double _xLap, _yLap;

		/// <summary>
		/// This node's cluster coefficient, which measures the proportion of neighbor nodes that are connected to each other.
		/// </summary>
		double _clusterCoeff;

		/// <summary>
		/// Graph coordinates in the cluster coefficient graph.
		/// </summary>
		double _xCC, _yCC;

		/// <summary>
		/// List of edges this vertex is a member of.
		/// </summary>
		ArrayList _inEdges;

		/// <summary>
		/// Records any vertices collapsed into this vertex
		/// </summary>
		ArrayList _childVertices;

		/// <summary>
		/// Whether or not this vertex has been collapsed into a parent vertex.
		/// </summary>
		bool _isChild;

		/// <summary>
		/// If this is a child vertex, this is a list of the vertices this vertex used to be connected to.
		/// </summary>
		ArrayList _wasConnectedTo;

		/// <summary>
		/// Adjacency eigenvector coordinates.
		/// </summary>
		EigenInformation _adjEigen;

		/// <summary>
		/// Laplacian eigenvector coordinates.
		/// </summary>
		EigenInformation _lapEigen;

		/// <summary>
		/// Normalized Laplacian eigenvector coordinates.
		/// </summary>
		EigenInformation _normLapEigen;

		/// <summary>
		/// Whether this vertex has component information calculated (cc must be large enough).
		/// </summary>
		public bool hasComponentInfo;

		/// <summary>
		/// The connected component this vertex belongs to.
		/// </summary>
		ConnectedComponent _connComp;

		/// <summary>
		/// Gets the specified adjacency eigenvector component.
		/// </summary>
		/// <param name="which">Which eigenvalue's eigenvector component to return.</param>
		/// <returns>The eigenvector component.</returns>
		public double getAdjacencyEigenvalue(int which)
		{
			return _connComp.adjEigenvectors[which, _connComp.Vertices.IndexOfKey(_id)];
		}

		/// <summary>
		/// Gets the specified laplacian eigenvector component.
		/// </summary>
		/// <param name="which">Which eigenvalue's eigenvector component to return.</param>
		/// <returns>The eigenvector component.</returns>
		public double getLaplacianEigenvalue(int which)
		{
			return _connComp.lapEigenvectors[which, _connComp.Vertices.IndexOfKey(_id)];
		}

		/// <summary>
		/// Gets the specified normalized laplacian eigenvector component.
		/// </summary>
		/// <param name="which">Which eigenvalue's eigenvector component to return.</param>
		/// <returns>The eigenvector component.</returns>
		public double getNormalizedLaplacianEigenvalue(int which)
		{
			return _connComp.normLapEigenvectors[which, _connComp.Vertices.IndexOfKey(_id)];
		}

		/// <summary>
		/// If this vertex is a Child Vertex, returns nodes this vertex used to be
		/// connected to.  If this vertex is not a Child Vertex, throws an error.
		/// </summary>
		public ArrayList WasConnectedTo
		{
			get
			{
				if (_isChild == false)
				{
					throw new Exception("Trying to get WasConnectedTo nodes for a vertex that is not a child.");
				}
				else
				{
					return _wasConnectedTo;
				}
			}
			set
			{
				if (_isChild == false)
				{
					throw new Exception("Trying to set WasConnectedTo nodes for a vertex that is not a child.");
				}
				else
				{
					_wasConnectedTo = value;
				}
			}
		}

		/// <summary>
		/// Property wrapper for _isChild
		/// </summary>
		public bool IsChild
		{
			get { return _isChild; }
			set { _isChild = value; }
		}

		/// <summary>
		/// Property wrapper for _childVertices
		/// </summary>
		public ArrayList ChildVertices
		{
			get { return _childVertices; }
			set { _childVertices = value; }
		}

		/// <summary>
		/// Property wrapper for _xLap
		/// </summary>
		public double XLap
		{
			get { return _xLap; }
			set { _xLap = value; }
		}

		/// <summary>
		/// Property wrapper for _yLap
		/// </summary>
		public double YLap
		{
			get { return _yLap; }
			set { _yLap = value; }
		}

		/// <summary>
		/// Property wrapper for _isPeripheral
		/// </summary>
		public bool IsPeripheral
		{
			get { return _isPeripheral; }
			set { _isPeripheral = value; }
		}

		/// <summary>
		/// Property wrapper for _minDist
		/// </summary>
		public double MinDist 
		{
			get { return _minDist; }
			set { _minDist = value; }
		}

		/// <summary>
		/// Property wrapper for _maxDist
		/// </summary>
		public double MaxDist 
		{
			get { return _maxDist; }
			set { _maxDist = value; }
		}

		/// <summary>
		/// Property wrapper for _avgDist
		/// </summary>
		public double AvgDist 
		{
			get { return _avgDist; }
			set { _avgDist = value; }
		}

		/// <summary>
		/// Property wrapper for _clusterCoeff
		/// </summary>
		public double ClusterCoeff 
		{
			get { return _clusterCoeff; }
			set { _clusterCoeff = value; }
		}

		/// <summary>
		/// Property wrapper for _connComp
		/// </summary>
		public ConnectedComponent ConnectedComponent 
		{
			get { return _connComp; }
			set { _connComp = value; }
		}

		/// <summary>
		/// Property wrapper for _adjEigen
		/// </summary>
		public EigenInformation AdjacencyEigenInformation 
		{
			get { return _adjEigen; }
			set { _adjEigen = value; }
		}

		/// <summary>
		/// Property wrapper for _lapEigen
		/// </summary>
		public EigenInformation LaplacianEigenInformation 
		{
			get { return _lapEigen; }
			set { _lapEigen = value; }
		}

		/// <summary>
		/// Property wrapper for _normLapEigen
		/// </summary>
		public EigenInformation NormalizedLaplacianEigenInformation 
		{
			get { return _normLapEigen; }
			set { _normLapEigen = value; }
		}

		/// <summary>
		/// Property wrapper for _isChemical
		/// </summary>
		public bool isChemical 
		{
			get { return _isChemical; }
			set { _isChemical = value; }
		}

		/// <summary>
		/// Property wrapper for _label
		/// </summary>
		public string Label 
		{
			get { return _label; }
			set { _label = value; }
		}

		/// <summary>
		/// Property wrapper for _id
		/// </summary>
		public int ID 
		{
			get { return _id; }
			set { _id = value; }
		}

		/// <summary>
		/// Property wrapper for _xpos
		/// </summary>
		public double X 
		{
			get { return _xpos; }
			set { _xpos = value; }
		}

		/// <summary>
		/// Property wrapper for _ypos
		/// </summary>
		public double Y 
		{
			get { return _ypos; }
			set { _ypos = value; }
		}

		/// <summary>
		/// Property wrapper for _xCC
		/// </summary>
		public double xCC 
		{
			get { return _xCC; }
			set { _xCC = value; }
		}

		/// <summary>
		/// Property wrapper for _yCC
		/// </summary>
		public double yCC 
		{
			get { return _yCC; }
			set { _yCC = value; }
		}

		/// <summary>
		/// Property wrapper for _xAvgDist
		/// </summary>
		public double xAverageDistance
		{
			get { return _xAvgDist; }
			set { _xAvgDist = value; }
		}

		/// <summary>
		/// Property wrapper for _yAvgDist
		/// </summary>
		public double yAverageDistance
		{
			get { return _yAvgDist; }
			set { _yAvgDist = value; }
		}

		/// <summary>
		/// Property wrapper for _xdisp
		/// </summary>
		public double dispX 
		{
			get { return _xdisp; }
			set { _xdisp = value; }
		}

		/// <summary>
		/// Property wrapper for _ydisp
		/// </summary>
		public double dispY 
		{
			get { return _ydisp; }
			set { _ydisp = value; }
		}

		/// <summary>
		/// Property wrapper for _inEdges
		/// </summary>
		public ArrayList InEdges 
		{
			get { return _inEdges; }
		}

		/// <summary>
		/// Vertex constructor
		/// </summary>
		/// <param name="id">This vertex's ID number.</param>
		/// <param name="n">This vertex's label.</param>
		/// <param name="isChemical">Whether this vertex is a chemical.</param>
		public Vertex(int id, string n, bool isChemical) {
			Int64 numTicks = System.DateTime.Now.Ticks % 1000000;
			Random r = new Random(id + (int)numTicks);

			_id = id;
			_label = n;
			_isChemical = isChemical;
			_inEdges = new ArrayList();
			_xpos = r.NextDouble() - 0.5;
			_ypos = r.NextDouble() - 0.5;
			_xdisp = 0.0;
			_ydisp = 0.0;

			_childVertices = new ArrayList();

			_isChild = false;

			hasComponentInfo = false;

			marked = false;

			r = null;
		}

		/// <summary>
		/// Adds a reference to an edge.
		/// </summary>
		/// <param name="ed">The edge to which to add a reference.</param>
		public void addRefToEdge(Edge ed) {
			if (_inEdges.Contains(ed) == false) {
				_inEdges.Add(ed);
				_numRefs++;
			}
		}

		/// <summary>
		/// Removes a reference to an edge.
		/// </summary>
		/// <param name="ed">The edge to which to remove a reference.</param>
		public void removeRefToEdge(Edge ed) 
		{
			if (_inEdges.Contains(ed) == true) 
			{
				_inEdges.Remove(ed);
				_numRefs--;
			}
		}

		/// <summary>
		/// Returns a list of vertices this vertex is connected to.
		/// </summary>
		public ArrayList ConnectedTo {
			get {
				ArrayList results = new ArrayList();
				
				foreach(Edge e in this._inEdges) {
					if(e.v1.Label == this.Label) {
						results.Add(e.v2);
					} else {
						results.Add(e.v1);
					}
				}

				return results;
			}
		}
	}
	

	/// <summary>
	/// An edge between two nodes in a graph
	/// </summary>
	public class Edge {
		/// <summary>
		/// Vertex one in the edge.
		/// </summary>
		public Vertex v1;

		/// <summary>
		/// Vertex two in the edge.
		/// </summary>
		public Vertex v2;

		/// <summary>
		/// String type of this edge...arbitrarily programmer-defined, but not used yet.
		/// </summary>
		string _type;

		/// <summary>
		/// Property wrapper for _type
		/// </summary>
		public string Type {
			get { return _type; }
			set { _type = value; }
		}

		/// <summary>
		/// Should be .ToString()
		/// </summary>
		/// <returns>String representing this edge.</returns>
		public string Display() {
			return v1.Label + "/" + v2.Label;
		}

		/// <summary>
		/// Edge constructor.
		/// 
		/// Note that the vertices have references added.
		/// </summary>
		/// <param name="node1">First node member of this edge.</param>
		/// <param name="node2">Second node member of this edge.</param>
		/// <param name="type">String type of this vertex</param>
		public Edge(Vertex node1, Vertex node2, string type) {
			v1 = node1;
			v2 = node2;

			if (v1.ConnectedTo.Contains(v2) == false)
			{
				v1.addRefToEdge(this);
			}

			if (v2.ConnectedTo.Contains(v1) == false)
			{
				v2.addRefToEdge(this);
			}

			_type = type;
		}
	}


	/// <summary>
	/// This class is the C# representation of the chemical interaction graph.
	/// It contains vertices, edges, and a collection of SubGraphs that are
	/// the connected components of the graph.
	/// NOTE that addVertex/addEdge does NOT update the adj matrices of the subgraphs
	///      since it would be so expensive.  This is done in .UpdateComponents()
	/// To calculate eigenspectra, first run .UpdateComponents() then run .CalculateVertexData() on a component.
	/// </summary>
	public class ChemGraph {
		/// <summary>
		/// The vertex array that goes along with this adjacency matrix
		/// Access .Component to find a vertex's subgraph, but only if you've run .UpdateComponents()!
		/// </summary>
		Hashtable _vertices;

		/// <summary>
		/// The list of edges in this graph.
		/// </summary>
		ArrayList _edges;

		/// <summary>
		///  A collection of ConnectedComponent objects that comprise this Graph.
		/// </summary>
		ArrayList _components;

		/// <summary>
		/// List of files currently being used, for housekeeping.
		/// </summary>
		ArrayList _files;

		/// <summary>
		/// Whether or not this graph is bipartite.  Unused?
		/// </summary>
		public bool isBipartite;

		/// <summary>
		/// This graph's associated web class.  For accessing .Trace() and stuff.
		/// </summary>
		DataAnalyzer _da;

		/// <summary>
		/// The current display type of this graph, normal or laplacianLLE.
		/// </summary>
		displayType _display;

		/// <summary>
		/// Property wrapper for _display.
		/// </summary>
		public displayType DisplayType
		{
			get { return _display; }
			set { _display = value; }
		}

		/// <summary>
		/// Property wrapper for _da.
		/// </summary>
		public DataAnalyzer da 
		{
			get { return _da; }
		}

		/// <summary>
		/// Returns the number of vertices in this graph.
		/// </summary>
		public int NumVertices {
			get { return _vertices.Count; }
		}

		/// <summary>
		/// Returns the number of edges in this graph.
		/// </summary>
		public int NumEdges {
			get { return _edges.Count; }
		}

		/// <summary>
		/// Property wrapper for _vertices.
		/// </summary>
		public Hashtable Vertices {
			get { return _vertices; }
		}

		/// <summary>
		/// Property wrapper for _edges.
		/// </summary>
		public ArrayList Edges {
			get { return _edges; }
		}

		/// <summary>
		/// Property wrapper for _components.
		/// </summary>
		public ArrayList Components {
			get { return _components; }
		}

		/// <summary>
		/// Returns the number of connected components that comprise this graph.
		/// </summary>
		public int NumComponents {
			get { return _components.Count; }
		}

		/// <summary>
		/// Gets a vertex by ID.
		/// </summary>
		/// <param name="ID">The ID number of the vertex you want.</param>
		/// <returns>The vertex desired.</returns>
		public Vertex getVertex(int ID) {
			return (Vertex)_vertices[ID];
		}

		/// <summary>
		/// Adds a file to _files for housekeeping later.
		/// </summary>
		/// <param name="s">The path of the file.</param>
		public void addFile(string s) {
			_files.Add(s);
		}

		/// <summary>
		/// Constructor for a ChemGraph.
		/// </summary>
		/// <param name="size">The expected size of the graph (non-binding).</param>
		/// <param name="da">The associated web page class.</param>
		public ChemGraph(int size, DataAnalyzer da) {
			//_adj = new double[size,size]; // not until we need it
			_vertices = new Hashtable(size);
			_edges = new ArrayList();
			_files = new ArrayList();
			isBipartite = true;
			_da = da;
		}

		/// <summary>
		/// Destructor -- cleans up files.
		/// </summary>
		~ChemGraph() {
			foreach(string s in _files) {
				try
				{
					File.Delete(s);
				}
				catch
				{
					// forget it...it'll be cleaned up later in PageLoad
				}
			}
		}

		/// <summary>
		/// Deletes files in the file queue.
		/// </summary>
		public void deleteFiles() {
			ArrayList toDelete = new ArrayList();

			foreach(string s in _files) {
				toDelete.Add(s);
			}

			foreach(string s in toDelete) {
				try {
					File.Delete(s);
					_files.Remove(s);
				} catch {
					// Wait for later destruction
				}
			}
		}

		/// <summary>
		/// Updates connected component information (to be run after graph building once)
		/// NOTE: This has been changed from DFS to BFS to stop stack overflows
		/// </summary>
		public void UpdateComponents() {
			DataAnalyzer da = new DataAnalyzer();
			da.Trace.Write("UpdateComponents", "Beginning...");

			_components = new ArrayList();

			foreach(DictionaryEntry de in _vertices) {
				Vertex v = (Vertex)(de.Value);

				if (v.marked == false) {
					// BFS
					ConnectedComponent cc = new ConnectedComponent();
					
					//
					// Set up BFS starting at node curVertex
					//
					Queue q = new Queue();
					v.marked = true;
					q.Enqueue(v);

					//
					// Do the BFS
					//
					while (q.Count > 0)
					{
						Vertex u = (Vertex)q.Dequeue();
						cc.addVertex(u);

						foreach (Vertex w in u.ConnectedTo)
						{
							if (w.marked == false)
							{
								w.marked = true;
								q.Enqueue(w);
							}
						}
					}

					//
					// Done with BFS, do accounting
					//
					cc.createAdjMatrix(cc.Vertices.Count);
					cc.Graph = this;

					_components.Add(cc);
				} else {
					// Do nothing
				}
			}

			foreach(ConnectedComponent c in _components) {
				//da.Trace.Write("UpdateComponents", "found component");
				if (c.NumVertices > 3) {
					da.Trace.Write("UpdateComponents", "component large enough, calculating info");
					c.calculateVertexComponentInformation();
				}
			}

			da.Trace.Write("UpdateComponents", "Ending...");
		}

		/// <summary>
		/// Collapses an array of nodes into a parent node
		/// </summary>
		/// <param name="nodes">The ArrayList of nodes to collapse.</param>
		/// <returns>The new parent vertex</returns>
		public Vertex collapseNodes(string name, ArrayList nodes)
		{
			if (nodes.Count < 2)
			{
				throw new Exception("ChemGraph.collapseNodes: Cannot collapse less than two nodes into a parent node.");
			}

			//
			// Don't allow sub-collapsing for now
			//
			foreach(Vertex v in nodes)
			{
				if (v.ChildVertices.Count > 0)
				{
					throw new Exception("ChemGraph.collapseNodes: Cannot sub-collapse nodes.");
				}
			}

			//
			// Make sure nodes are in the same connected component
			//
			ConnectedComponent firstcc = ((Vertex)nodes[0]).ConnectedComponent;
			foreach(Vertex v in nodes)
			{
				if (v.ConnectedComponent != firstcc)
				{
					throw new Exception("ChemGraph.collapseNodes: Cannot collapse nodes that aren't in the same connected component.");
				}
			}

			//
			// Create the new vertex, add into _vertices
			//
			int maxID = 0;

			foreach(int key in _vertices.Keys)
			{
				if (key > maxID) maxID = key;
			}

			Vertex newguy = new Vertex(maxID + 1, name, false);
			newguy.ConnectedComponent = ((Vertex)nodes[0]).ConnectedComponent;
			_vertices.Add(newguy.ID, newguy);

			//
			// Remove old vertices from _vertices
			//
			foreach(Vertex v in nodes)
			{
				_vertices.Remove(v.ID);
			}

			//
			// Swap edges, making sure there are no self-edges, and storing old info into WasConnectedTo
			//
			foreach(Vertex v in nodes)
			{
				v.IsChild = true;
				v.WasConnectedTo = new ArrayList();

				//
				// Set WasConnectedTo
				//
				foreach(Edge e in v.InEdges)
				{
					if (e.v1 == v)
					{
						v.WasConnectedTo.Add(e.v2);
					}
					else
					{
						v.WasConnectedTo.Add(e.v1);
					}
				}

				//
				// Swap edges and remove self-edges
				//
				ArrayList edgesToDelete = new ArrayList();
				foreach(Edge e in v.InEdges)
				{
					// If this is a self-edge, remove it
					// This must be done AFTER we set WasConnectedTo, cause
					// this removes ref information and we wouldn't get the
					// right WasConnectedTo otherwise
					if ((e.v1 == newguy) || (e.v2 == newguy) || 
						(newguy.ConnectedTo.Contains(e.v1)) || (newguy.ConnectedTo.Contains(e.v2)))
					{
						edgesToDelete.Add(e);
						continue;
					}

					// Change edge's information
					// Note that we don't have to change v.ConnectedTo, since that's dynamically updates
					if (e.v1 == v)
					{
						e.v1 = newguy;
						e.v1.removeRefToEdge(e);
					}
					else
					{
						e.v2 = newguy;
						e.v2.removeRefToEdge(e);
					}

					newguy.addRefToEdge(e);
				}

				foreach(Edge e in edgesToDelete)
				{
					this.removeEdge(e.v1, e.v2);
					newguy.removeRefToEdge(e);
				}
			}

			//
			// Update connected component
			//
			ConnectedComponent cc = newguy.ConnectedComponent;
			
			cc.Vertices.Add(newguy.ID, newguy);
			foreach(Vertex v in nodes)
			{
				cc.Vertices.Remove(v.ID);
			}

			cc.createAdjMatrix(newguy.ConnectedComponent.Vertices.Count);
			cc.calculateVertexComponentInformation();

			//
			// Update vertex LB
			//
			da.updateVertexListBox();

			return newguy;
		}

		/// <summary>
		/// Performs a dfs on this graph.
		/// </summary>
		/// <param name="v">Vertex to dfs on.</param>
		/// <param name="vertices">ArrayList to add found vertices to.</param>
		private void dfs(Vertex v, ArrayList vertices) {
			foreach(Vertex u in v.ConnectedTo) {
				if (u.marked == false) {
					vertices.Add(u);
					u.marked = true;
					dfs(u, vertices);
				}
			}
		}

		/// <summary>
		/// Adds a vertex to this graph (and created the vertex, too).
		/// OVERLOADED
		/// </summary>
		/// <param name="id">The id number of the new node.</param>
		/// <param name="str">The label of the new node.</param>
		/// <param name="isChemical">Whether the new node is a chemical or not.</param>
		public void addVertex(int id, string str, bool isChemical) {
			_vertices.Add(id, new Vertex(id, str, isChemical));
		}

		/// <summary>
		/// Adds a vertex to this graph by reference.
		/// OVERLOADED
		/// </summary>
		/// <param name="v1">The vertex to add.</param>
		public void addVertex(Vertex v1) {
			_vertices.Add(v1.ID, v1);
		}

		/// <summary>
		/// Adds an edge to this graph.
		/// </summary>
		/// <param name="v1">The first vertex in this edge.</param>
		/// <param name="v2">The second vertex in this edge.</param>
		/// <param name="str">This edge's string type (not necessary).</param>
		public void addEdge(Vertex v1, Vertex v2, string str) {
			foreach(Edge e in _edges) 
			{
				if (((e.v1 == v1) && (e.v2 == v2)) || ((e.v1 == v2) && (e.v2 == v1))) 
				{
					return; // already in there
				}
			}

			_edges.Add(new Edge(v1, v2, str));
		}

		/// <summary>
		/// Add an edge to this graph (SEARCHES FOR VERTICES)
		/// </summary>
		/// <param name="id1">the ID of one of the vertices in the edge</param>
		/// <param name="id2">the ID of the other vertex in the edge</param>
		/// <param name="str">the type of the edge</param>
		public void addEdge(int id1, int id2, string str) {
			Vertex v1;
			Vertex v2;

			v1 = (Vertex) _vertices[id1];
			v2 = (Vertex) _vertices[id2];

			if ((v1 != null) && (v2 != null) && (str != null)) {
				foreach(Edge e in _edges) {
					if (((e.v1 == v1) && (e.v2 == v2)) || ((e.v1 == v2) && (e.v2 == v1))) {
						return; // already in there
					}
				}

				_edges.Add(new Edge(v1, v2, str));
			} else {
				throw new ArgumentException("Either one of the IDs you passed isn't paired with a vertex, or the edge type was null.");
			}
		}

		/// <summary>
		/// Remove an edge containing the two specified vertices.  UpdateComponents() must be called outside of this function.
		/// </summary>
		/// <param name="v1">First vertex.</param>
		/// <param name="v2">Second vertex.</param>
		public void removeEdge(Vertex v1, Vertex v2)
		{
			Edge toDelete = null;

			foreach (Edge e in _edges)
			{
				if (((e.v1 == v1) && (e.v2 == v2)) ||
					((e.v2 == v1) && (e.v1 == v2)))
				{
					toDelete = e;
				}
			}

			if (toDelete != null)
			{
				_edges.Remove(toDelete);
				toDelete.v1.removeRefToEdge(toDelete);
				toDelete.v2.removeRefToEdge(toDelete);
			}

			//
			// DON'T ALWAYS NEED TO UPDATE COMPONENTS, so leave to calling function
			//
		}

		/// <summary>
		/// Remove an edge containing the two specified vertices.  UpdateComponents() must be called outside of this function.
		/// </summary>
		/// <param name="id1">First vertex's ID.</param>
		/// <param name="id2">Second vertex's ID.</param>
		public void removeEdge(int id1, int id2)
		{
			Edge toDelete = null;
			Vertex v1 = (Vertex)_vertices[id1];
			Vertex v2 = (Vertex)_vertices[id2];

			foreach (Edge e in _edges)
			{
				if (((e.v1 == v1) && (e.v2 == v2)) ||
					((e.v2 == v1) && (e.v1 == v2)))
				{
					toDelete = e;
				}
			}

			if (toDelete != null)
			{
				_edges.Remove(toDelete);
			}

			//
			// DON'T ALWAYS NEED TO UPDATE COMPONENTS, so leave to calling function
			//
		}
		
	}


	/// <summary>
	/// Represents a connected component of the graph.  This is where we calculate a vertex's spectral info.
	/// See Vertex.ComponentInfo to access a vertex's spectral information.
	/// This class holds the SparseMatrix adjacency matrix.
	/// </summary>
	public class ConnectedComponent {

		/// <summary>
		/// The adjacency matrix.
		/// </summary>
		SparseMatrix _adj;

		/// <summary>
		/// The vertices in this graph.  This is a duplication of data in its parent's ChemGraph, which isn't good.
		/// </summary>
		SortedList _vertices;

		/// <summary>
		/// The graph this cc belongs to.
		/// </summary>
		ChemGraph _graph;
		
		/// <summary>
		/// The shortest path matrix for this cc.  [i,j] represents the shortest path from cc.getByIndex(i) to cc.getByIndex(j).
		/// Why is this a double array?
		/// </summary>
		double[,] _shortestPath;

		/// <summary>
		/// The average diameter of this cc.
		/// </summary>
		double _avgDiameter;

		/// <summary>
		/// The URL location of this cc's laplacian LLE graph.
		/// </summary>
		string _laplacianGraphLocation;

		/// <summary>
		/// This cc's minimum average path length (across all vertices).
		/// </summary>
		double _minAvgPathLength;

		/// <summary>
		/// This cc's maximum average path length (across all vertices).
		/// </summary>
		double _maxAvgPathLength;

		/// <summary>
		/// The minimum degree of a vertex across all vertices in this cc.
		/// </summary>
		int _minDegree;

		/// <summary>
		/// The maximum degree of a vertex across all vertices in this cc.
		/// </summary>
		int _maxDegree;

		/// <summary>
		/// This cc's adjacency eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] adjEigens;

		/// <summary>
		/// This cc's laplacian eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] lapEigens;

		/// <summary>
		/// This cc's normalized laplacian eigenvalues.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] normLapEigens;

		/// <summary>
		/// This cc's adjacency eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] adjIVP;

		/// <summary>
		/// This cc's laplacian eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] lapIVP;

		/// <summary>
		/// This cc's normalized laplacian eigenvalues' Inverse Participation Ratio.  Filled in in .calculateEigenvalues().
		/// </summary>
		public double[] normLapIVP;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. adjEigenvectors[0,x] is eig 1)
		/// </summary>
		public double[,] adjEigenvectors;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. lapEigenvectors[0,x] is eig 1)
		/// </summary>
		public double[,] lapEigenvectors;

		/// <summary>
		/// These matrices hold the eigenvectors HORIZONTALLY (i.e. normLapEigenvectors[0,x] is eig 1)
		/// </summary>
		public double[,] normLapEigenvectors;

		/// <summary>
		/// Property wrapper for _laplacianGraphLocation.
		/// </summary>
		public string LaplacianGraphLocation
		{
			get { return _laplacianGraphLocation; }
			set { _laplacianGraphLocation = value; }
		}

		/// <summary>
		/// Property wrapper for _minAvgPathLength.
		/// </summary>
		public double MinAveragePathLength 
		{
			get { return _minAvgPathLength; }
			set { _minAvgPathLength = value; }
		}

		/// <summary>
		/// Property wrapper for _maxAvgPathLength.
		/// </summary>
		public double MaxAveragePathLength
		{
			get { return _maxAvgPathLength; }
			set { _maxAvgPathLength = value; }
		}

		/// <summary>
		/// Property wrapper for _avgDiameter.
		/// </summary>
		public double AverageDiameter 
		{
			get { return _avgDiameter; }
		}

		/// <summary>
		/// Property wrapper for _adj.
		/// </summary>
		public SparseMatrix adj 
		{
			get { return _adj; }
		}

		/// <summary>
		/// Property wrapper for _graph.
		/// </summary>
		public ChemGraph Graph {
			get { return _graph; }
			set { _graph = value; }
		}

		/// <summary>
		/// Property wrapper for _vertices
		/// </summary>
		public SortedList Vertices {
			get { return _vertices; }
		}

		/// <summary>
		/// Returns the number of vertices in this cc.
		/// </summary>
		public int NumVertices {
			get { return _vertices.Count; }
		}

		/// <summary>
		/// Dynamically finds the edges in this cc (kind of nasty implementation).
		/// </summary>
		public ArrayList Edges 
		{
			// this is a pretty nasty implementation, but whatever
			get
			{
				ArrayList result = new ArrayList();

				foreach (Edge e in Graph.Edges)
				{
					if (e.v1.ConnectedComponent == this)
					{
						result.Add(e);
					}
				}

				return result;
			}
		}

		/// <summary>
		/// Constructs a ConnectedComponent.
		/// </summary>
		public ConnectedComponent() {
			_vertices = new SortedList();
			_minDegree = Int32.MaxValue;
			_maxDegree = Int32.MinValue;
			_minAvgPathLength = double.MaxValue;
			_maxAvgPathLength = double.MinValue;
		}

		/// <summary>
		/// Finds (and sets) the minimum degree for this cc.
		/// </summary>
		public int MinDegree {
			get {
				if (_minDegree != Int32.MaxValue) {
					return _minDegree;
				} else {
					foreach(DictionaryEntry de in _vertices) {
						Vertex v = (Vertex)(de.Value);
						
						if (v.ConnectedTo.Count < _minDegree) {
							_minDegree = v.ConnectedTo.Count;
						}
					}

					return _minDegree;
				}
			}
		}

		/// <summary>
		/// Finds (and sets) the maximum degree for this cc.
		/// </summary>
		public int MaxDegree 
		{
			get {
				if (_maxDegree != Int32.MinValue) {
					return _maxDegree;
				} else {
					foreach(DictionaryEntry de in _vertices) {
						Vertex v = (Vertex)(de.Value);
						
						if (v.ConnectedTo.Count > _maxDegree) {
							_maxDegree = v.ConnectedTo.Count;
						}
					}

					return _maxDegree;
				}
			}
		}

		/// <summary>
		/// Adds a vertex to this cc.
		/// </summary>
		/// <param name="v">The vertex to add.</param>
		public void addVertex(Vertex v) {
			_vertices.Add(v.ID, v);
			v.ConnectedComponent = this;
		}

		/// <summary>
		/// Creates the adjacency matrix for this connected component.
		/// 
		/// NOTE that this matrix is 0-indexed and is NOT INDEXED BASED ON NODE ID.  You can reproduce the indexing scheme
		/// by a) doing a foreach on _vertices, or b) doing a .getByIndex(i) while for'ing from 0 to NumVertices
		/// </summary>
		/// <param name="size">The number of vertices in the graph, which is the rank of the adjacency matrix.</param>
		public void createAdjMatrix(int size) {
			_adj = new SparseMatrix(size);

			adjEigens = new double[size];
			lapEigens = new double[size];
			normLapEigens = new double[size];

			adjIVP = new double[size];
			lapIVP = new double[size];
			normLapIVP = new double[size];

			for (int j = 0; j < size; j++)
			{
				adjIVP[j] = 0.0;
				lapIVP[j] = 0.0;
				normLapIVP[j] = 0.0;
			}

			int i = 0;
			foreach(DictionaryEntry de in _vertices) {
				Vertex v = (Vertex)(de.Value);

				int j = 0;
				foreach(DictionaryEntry de2 in _vertices) {
					Vertex u = (Vertex)(de2.Value);

					if (v.ConnectedTo.Contains(u)) {
						_adj.setValue(i, j, 1);
					}

					j++;
				}

				i++;
			}

			//if (_vertices.Count > 200) {
			//	FileStream fs = File.Create("C:\\matrix.txt");
			//	StreamWriter sw = new StreamWriter(fs);

			//	for (int j = 0; j < _vertices.Count; j++) {
			//		string sb;
					
			//		sb = _adj.getValue(j, 0).ToString();

			//		for (int k = 1; k < _vertices.Count; k++) {
			//			sb += "," + _adj.getValue(j, k).ToString();
			//		}

			//		sw.WriteLine(sb);
			//	}

			//	sw.Close();
			//}
		}

		/// <summary>
		/// Does all the nifty calculations on this connected component.  Includes path length, eigeninformation,
		/// cluster coefficient, etc.  And it sets graph coordinates for vertices for these values.
		/// </summary>
		public void calculateVertexComponentInformation() {
			// Find the shortest path between every node
			findPathLengths();

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);
				v.AdjacencyEigenInformation = new EigenInformation(this._vertices.Count);
				v.LaplacianEigenInformation = new EigenInformation(this._vertices.Count);
				v.NormalizedLaplacianEigenInformation = new EigenInformation(this._vertices.Count);
			}

			// Calculate eigenvalues
			calculateEigenValues(EigenType.Adjacency); // adjacency matrix
			calculateEigenValues(EigenType.Laplacian); // laplacian
			calculateEigenValues(EigenType.NormalizedLaplacian); // normalized laplacian

			foreach(DictionaryEntry de in _vertices) {
				Vertex v = (Vertex)(de.Value);

				setClusterCoefficient(v);

				// Now turn on display
				v.hasComponentInfo = true;
			}

			// Set coordinates for each vertex's cluster coefficient
			setCCCoords();
			setPathLengthCoords();
			setEigenVectorCoords(EigenType.Adjacency);
			setEigenVectorCoords(EigenType.Laplacian);
			setEigenVectorCoords(EigenType.NormalizedLaplacian);
			calculateLaplacianGraphCoords();
		}

		/// <summary>
		/// Calculates graph coordinates in the Laplacian LLE graph for each vertex.
		/// </summary>
		private void calculateLaplacianGraphCoords()
		{
			double xScale = 1.0;
			double yScale = 1.0;

			double numVertices = Vertices.Count;

			//
			// Set original graph positions
			//
			foreach (DictionaryEntry de in Vertices)
			{
				Vertex v = (Vertex)de.Value;

				if (v.getLaplacianEigenvalue(1) > 0)
				{
					v.XLap = Math.Log10(v.getLaplacianEigenvalue(1));
				}
				else if (v.getLaplacianEigenvalue(1) == 0)
				{
					v.XLap = 0.0;
				}
				else // < 0
				{
					v.XLap = -1 * Math.Log10(-1 * v.getLaplacianEigenvalue(1));
				}

				if (v.getLaplacianEigenvalue(2) > 0)
				{
					v.YLap = Math.Log10(v.getLaplacianEigenvalue(2));
				}
				else if (v.getLaplacianEigenvalue(2) == 0)
				{
					v.YLap = 0.0;
				}
				else // < 0
				{
					v.YLap = -1 * Math.Log10(-1 * v.getLaplacianEigenvalue(2));
				}
			}

			//
			// Scale graph
			//
			// Find x, y scales
			double minX = Double.MaxValue;
			double maxX = Double.MinValue;
			double minY = Double.MaxValue;
			double maxY = Double.MinValue;
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v.XLap < minX) 
				{
					minX = v.XLap;
				}

				if (v.XLap > maxX) 
				{
					maxX = v.XLap;
				}

				if (v.YLap < minY) 
				{
					minY = v.YLap;
				}

				if (v.YLap > maxY) 
				{
					maxY = v.YLap;
				}
			}

			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.XLap -= (maxX + minX) / 2.0;
				v.YLap -= (maxY + minY) / 2.0;
			}
				
			xScale = 330 / (maxX - ((maxX + minX) / 2.0));
			yScale = 330 / (maxY - ((maxY + minY) / 2.0));

			// Scale vertices
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				v.XLap = v.XLap * xScale + 345;
				v.YLap = v.YLap * yScale + 345;
			}
		}

		/// <summary>
		/// Finds the average diameter of the graph based on shortest path information
		/// </summary>
		private void findAverageDiameter()
		{
			double numerator = 0.0;
			double denominator = (_vertices.Count)*(_vertices.Count) - _vertices.Count;

			for (int i = 0; i < _vertices.Count; i++)
			{
				for (int j = 0; j < _vertices.Count; j++)
				{
					numerator += _shortestPath[i,j];
				}
			}

			_avgDiameter = numerator / denominator;
		}

		/// <summary>
		/// This function runs a BFS on every node to determine the shortest path between every node
		/// (in the unweighted case, BFS finds shortest path length)
		/// http://www.csee.umbc.edu/courses/undergraduate/341/fall01/Lectures/Graphs/11
		/// </summary>
		private void findPathLengths() 
		{
			// Allocate shortest path matrix
			// This is indexed as _adj is indexed, from index of _vertices
			_shortestPath = new double[_vertices.Count, _vertices.Count];

			//
			// Perform BFS for each row in the graph
			// REMEMBER that adjacency matrix is indexed by index of _vertices!!
			//
			for (int i = 0; i < _vertices.Count; i++)
			{
				Vertex curVertex = (Vertex)_vertices.GetByIndex(i);

				//
				// Set up BFS starting at node curVertex
				//
				Queue q = new Queue();

				// Set every distance to infinity
				for (int j = 0; j < _vertices.Count; j++)
				{
					_shortestPath[i,j] = double.PositiveInfinity;
				}

				q.Enqueue(curVertex);
				
				// Set path from i to i to 0
				_shortestPath[i,i] = 0;

				//
				// Do the BFS
				//
				while (q.Count > 0)
				{
					Vertex u = (Vertex)q.Dequeue();
					int indexOfU = _vertices.IndexOfValue(u);

					foreach (Vertex w in u.ConnectedTo)
					{
						int indexOfW = _vertices.IndexOfValue(w);

						if (_shortestPath[i,indexOfW] == double.PositiveInfinity)
						{
							_shortestPath[i,indexOfW] = _shortestPath[i,indexOfU] + 1;
							q.Enqueue(w);
						}
					}
				}

				// set node min,max,avg info
				curVertex.MinDist = double.MaxValue;
				curVertex.MaxDist = double.MinValue;
				curVertex.AvgDist = 0.0;
				for (int j = 0; j < _vertices.Count; j++)
				{
					if ((i != j) && (_shortestPath[i,j] < curVertex.MinDist)) curVertex.MinDist = _shortestPath[i,j];
					if ((i != j) && (_shortestPath[i,j] > curVertex.MaxDist)) curVertex.MaxDist = _shortestPath[i,j];
					curVertex.AvgDist += (_shortestPath[i,j] / ((double)_vertices.Count - 1));
				}
				if (curVertex.AvgDist < this._minAvgPathLength) this._minAvgPathLength = curVertex.AvgDist;
				if (curVertex.AvgDist > this._maxAvgPathLength) this._maxAvgPathLength = curVertex.AvgDist;
			}

			// Set the average diameter -- MUST COME AFTER findShortestPaths()!!!
			findAverageDiameter();
		}

		/// <summary>
		/// Finds and sets the average path length graph coordinates for each vertex.
		/// </summary>
		public void setPathLengthCoords()
		{
			int w = 500;
			int h = 300;

			foreach(DictionaryEntry de in _vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (this.MaxDegree == this.MinDegree)
				{
					v.xAverageDistance = 30.0 + .5 * ((double)w - 60.0);
				}
				else
				{
					v.xAverageDistance = 30.0 + (1.0 - (((double)(this.MaxDegree - v.ConnectedTo.Count)) / 
						((double)(this.MaxDegree - this.MinDegree)))) * ((double)(w - 60));
				}

				if (this.MaxAveragePathLength == this.MinAveragePathLength)
				{
					v.yAverageDistance = 30.0 + .5 * ((double)h - 60.0) - 2.0;
				}
				else
				{
					v.yAverageDistance = 30.0 + (((double)(this.MaxAveragePathLength - v.AvgDist)) / 
						((double)(this.MaxAveragePathLength - this.MinAveragePathLength))) * ((double)(h - 60)) - 2.0;
				}
			}
		}

		/// <summary>
		/// Sets Y-VALUE of eigenvalue coordinates.  Doesn't set x-value because that's determined by SORT PREFERENCE.
		/// </summary>
		private void setEigenVectorCoords(EigenType type) 
		{
			//int w = 500;
			int h = 300;

			double minVal = Double.MaxValue;
			double maxVal = Double.MinValue;

			for (int i = 0; i < 4; i++) // to iterate through EigenInformation.
			{
				minVal = Double.MaxValue;
				maxVal = Double.MinValue;

				//
				// Find min/maxval
				//
				foreach(DictionaryEntry de in _vertices) 
				{
					Vertex v = (Vertex)(de.Value);
					switch (type)
					{
						case EigenType.Adjacency:
							if (v.getAdjacencyEigenvalue(i) < minVal)
								minVal = v.getAdjacencyEigenvalue(i);
							if (v.getAdjacencyEigenvalue(i) > maxVal)
								maxVal = v.getAdjacencyEigenvalue(i);
							break;
						case EigenType.Laplacian:
							if (v.getLaplacianEigenvalue(i) < minVal)
								minVal = v.getLaplacianEigenvalue(i);
							if (v.getLaplacianEigenvalue(i) > maxVal)
								maxVal = v.getLaplacianEigenvalue(i);
							break;
						case EigenType.NormalizedLaplacian:
							if (v.getNormalizedLaplacianEigenvalue(i) < minVal)
								minVal = v.getNormalizedLaplacianEigenvalue(i);
							if (v.getNormalizedLaplacianEigenvalue(i) > maxVal)
								maxVal = v.getNormalizedLaplacianEigenvalue(i);
							break;
					}
				}

				//
				// Set coordinates (Y-VALUE ONLY!!! SEE SUMMARY!)
				//
				foreach(DictionaryEntry de in _vertices)
				{
					Vertex v = (Vertex)(de.Value);
					switch (type)
					{
						case EigenType.Adjacency:
							v.AdjacencyEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.AdjacencyEigenInformation.y[i] = 30.0 + ((maxVal - v.getAdjacencyEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.AdjacencyEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
						case EigenType.Laplacian:
							v.LaplacianEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.LaplacianEigenInformation.y[i] = 30.0 + ((maxVal - v.getLaplacianEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.LaplacianEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
						case EigenType.NormalizedLaplacian:
							v.NormalizedLaplacianEigenInformation.x[i] = 0;
							if ((maxVal - minVal) > 0)
							{
								v.NormalizedLaplacianEigenInformation.y[i] = 30.0 + ((maxVal - v.getNormalizedLaplacianEigenvalue(i)) 
									/ (maxVal - minVal)) * ((double)(h - 60)) - 2.0;
							}
							else
							{
								v.NormalizedLaplacianEigenInformation.y[i] = 30.0 + ((double)(h - 60))*(0.5) - 2.0;
							}
							break;
					}
				}
			}
		}
		
		/// <summary>
		/// Sets Cluster Coefficient coordinates.
		/// </summary>
		private void setCCCoords() {
			int w = 500;
			int h = 300;

			double minCC = 0.0;
			double maxCC = 1.0;

			foreach(DictionaryEntry de in _vertices) {
				Vertex v = (Vertex)(de.Value);

				if (Double.IsNaN(v.ClusterCoeff) == false) {
					if (this.MaxDegree == this.MinDegree)
					{
						v.xCC = 30.0 + .5 * ((double)w - 60.0);
					}
					else
					{
						v.xCC = 30.0 + (1.0 - (((double)(this.MaxDegree - v.ConnectedTo.Count)) / ((double)(this.MaxDegree - this.MinDegree)))) * ((double)(w - 60));
					}
					
					if (maxCC == minCC)
					{
						v.yCC = 30.0 + .5 * ((double)h-60) - 2.0;
					}
					else
					{
						v.yCC = 30.0 + (((double)(maxCC - v.ClusterCoeff)) / ((double)(maxCC - minCC))) * ((double)(h - 60)) - 2.0;
					}
				}
			}
		}

		/// <summary>
		/// Finds (and sets) the cluster coefficient for a vertex.
		/// </summary>
		/// <param name="v">The vertex whose cluster coefficient to calculate.</param>
		private void setClusterCoefficient(Vertex v) {
			double result = 0.0;
			int numLinks = 0;

			ArrayList neighbors = v.ConnectedTo;
			foreach(Vertex u in neighbors) {
				foreach(Vertex w in u.ConnectedTo) {
					//if (u == w) continue;
					if (neighbors.Contains(w)) {
						numLinks++;
					}
				}
			}

			if ((numLinks % 2) != 0) {
				v.ConnectedComponent.Graph.da.Response.Write("Error: numLinks not even<br><br>");
			}
			numLinks = numLinks / 2;

			result = (2.0 * ((double)numLinks)) / ((double)(neighbors.Count*(neighbors.Count - 1)));

			v.ConnectedComponent.Graph.da.Trace.Write("setClusterCoefficient", "C(" + v.Label + ") = " + result.ToString() + "; numLinks = " + numLinks.ToString() + "; NumNeighbors = " + neighbors.Count.ToString());

			v.ClusterCoeff = result;
		}

		/// <summary>
		/// Calculates eigenvalue information for this connected component.
		/// 
		/// WARNING: This function will use MASSIVE AMOUNTS OF MEMORY for large matrices
		///          For example, a matrix of order 426 uses 1.4MB, while a matrix of order
		///          5000 uses 200MB.  This is a LOT of memory (at least in 2003).
		/// </summary>
		private void calculateEigenValues(EigenType type)
		{
			string jobz = "V";
			string uplo = "U";
			int n = _vertices.Count;
			int lda = n;
			double[] work = new double[3*n];
			int lwork = 3*n;
			int info;

			try
			{
				switch (type) 
				{
					case EigenType.Adjacency:
						adjEigenvectors = new double[n,n];
						break;
					case EigenType.Laplacian:
						lapEigenvectors = new double[n,n];
						break;
					case EigenType.NormalizedLaplacian:
						normLapEigenvectors = new double[n,n];
						break;
				}
			}
			catch
			{
				throw new Exception("Ran out of memory storing eigenvectors.");
			}

			//
			// Fill in the ap matrix (the matrix we're doing calculations on)
			//
			// if UPLO = 'U', AP(i + (j-1)*j/2) = A(i,j) for 1<=i<=j;
			// REMEMBER that this version of Fortran flips (i,j) in square matrices
			for (int i = 0; i < n; i++) 
			{
				for (int j = 0; j < n; j++) 
				{
					if (type == EigenType.Adjacency) 
					{
						// ADJACENCY MATRIX
						adjEigenvectors[i,j] =  _adj.getValue(i, j);
					}
					else if (type == EigenType.Laplacian) 
					{
						// LAPLACIAN
						Vertex v = (Vertex)_vertices.GetByIndex(i);
						Vertex u = (Vertex)_vertices.GetByIndex(j);

						if (i == j) 
						{
							lapEigenvectors[i,j] = v.ConnectedTo.Count;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							lapEigenvectors[i,j] = -1.0;
						} 
						else 
						{
							lapEigenvectors[i,j] = 0;
						}
					}
					else if (type == EigenType.NormalizedLaplacian)
					{
						// NORMALIZED LAPLACIAN
						Vertex v = (Vertex)_vertices.GetByIndex(i);
						Vertex u = (Vertex)_vertices.GetByIndex(j);

						if (i == j) 
						{
							normLapEigenvectors[i,j] = 1;
						} 
						else if (v.ConnectedTo.Contains(u)) 
						{
							normLapEigenvectors[i,j] = - (1.0 / (Math.Sqrt(v.ConnectedTo.Count * u.ConnectedTo.Count)));
						} 
						else 
						{
							normLapEigenvectors[i,j] = 0;
						}
					}
				}
			}

			//
			// Call the fortran DLL
			//
			switch (type) 
			{
				case EigenType.Adjacency:
					Eigensolver.eigenstub(jobz, uplo, n, adjEigenvectors, lda, adjEigens, work, lwork, out info);
					break;
				case EigenType.Laplacian:
					Eigensolver.eigenstub(jobz, uplo, n, lapEigenvectors, lda, lapEigens, work, lwork, out info);
					break;
				case EigenType.NormalizedLaplacian:
					Eigensolver.eigenstub(jobz, uplo, n, normLapEigenvectors, lda, normLapEigens, work, lwork, out info);
					break;
			}

			//
			// Truncate eigenvalues
			//
			for (int i = 0; i < n; i++)
			{
				if (type == EigenType.Adjacency) 
				{
					// ADJACENCY
					if (Math.Abs(adjEigens[i]) < 0.0000000000001)
					{
						adjEigens[i] = 0.0;
					}
				} 
				else if (type == EigenType.Laplacian)
				{
					// LAPLACIAN
					if (Math.Abs(lapEigens[i]) < 0.0000000000001)
					{
						lapEigens[i] = 0.0;
					}
				} 
				else if (type == EigenType.NormalizedLaplacian) 
				{
					// NORMALIZED LAPLACIAN
					if (Math.Abs(normLapEigens[i]) < 0.0000000000001)
					{
						normLapEigens[i] = 0.0;
					}
				}
			}

			//
			// Normalize eigenvectors
			//
			/* THIS APPEARS UNNECESSARY
			for (int i = 0; i < n; i++) 
			{
				double sumSquares = 0.0;
				for (int j = 0; j < n; j++) 
				{
					switch (type) 
					{
						case EigenType.Adjacency:
							sumSquares += Math.Pow(a[i,j], 2.0);
							break;
						case EigenType.Laplacian:
							sumSquares += Math.Pow(a[i,j], 2.0);
							break;
						case EigenType.NormalizedLaplacian:
							sumSquares += Math.Pow(a[i,j], 2.0);
							break;
					}
				}

				double length = Math.Sqrt(sumSquares);
				for (int j = 0; j < n; j++) 
				{
					a[i,j] = a[i,j] / length;
				}
			}
			*/

			// Guide: 
			//		In the inner loop, i will be your eigenvector (no, not me), while j will be the node num to which it belongs (A[j,i]).
			//		So on the inside, we use i for which eigenvector we pair this to.  "type" is used for which eigeninformation.
			//
			for (int i = 0; i < n; i++)
			{
				for (int j = 0; j < n; j++) 
				{
					switch (type) 
					{
						case EigenType.Adjacency:
							this.adjIVP[i] += Math.Pow(adjEigenvectors[i,j], 4.0);
							break;
						case EigenType.Laplacian:
							this.lapIVP[i] += Math.Pow(lapEigenvectors[i,j], 4.0);
							break;
						case EigenType.NormalizedLaplacian:
							this.normLapIVP[i] += Math.Pow(normLapEigenvectors[i,j], 4.0);
							break;
					}
				}
			}

			for (int i = 0; i < n; i++)
			{
				switch (type) 
				{
					case EigenType.Adjacency:
						this.adjIVP[i] = -1.0 * Math.Log10(this.adjIVP[i]);
						break;
					case EigenType.Laplacian:
						this.lapIVP[i] = -1.0 * Math.Log10(this.lapIVP[i]);
						break;
					case EigenType.NormalizedLaplacian:
						this.normLapIVP[i] = -1.0 * Math.Log10(this.normLapIVP[i]);
						break;
				}
			}
		}

		/// <summary>
		/// Draws the laplacian LLE graph.
		/// </summary>
		/// <param name="curVertex">The current vertex.</param>
		/// <param name="graphDir">The graph directory to place the file in.</param>
		public void drawLaplacianGraph(Vertex curVertex, string graphDir) 
		{
			Pen whitePen = new Pen(Color.White, 1);
			Pen grayPen = new Pen(Color.Gray, 1);
			Pen blackPen = new Pen(Color.Black, 1);
			Pen bigBlackPen = new Pen(Color.Black, 2);
			Pen dgPen = new Pen(Color.DarkGray, 1);
			Pen pinkPen = new Pen(Color.Pink, 1);
			Pen orangePen = new Pen(Color.Orange, 2);
			Pen redPen = new Pen(Color.Red, 1);
			Brush whiteBrush = new SolidBrush(Color.White);
			Brush grayBrush = new SolidBrush(Color.Gray);
			Brush redBrush = new SolidBrush(Color.FromArgb(255, 51, 0));
			Brush greenBrush = new SolidBrush(Color.Green);
			Brush blackBrush = new SolidBrush(Color.Black);
			Brush orangeBrush = new SolidBrush(Color.Orange);
			Brush pinkBrush = new SolidBrush(Color.Pink);
			Brush darkGreenBrush = new SolidBrush(Color.DarkGreen);
			Brush siennaBrush = new SolidBrush(Color.FromArgb(0, 51, 153));

			int w = 700;
			int h = 700;
			Bitmap pic = new Bitmap(w, h);
			Graphics pad = Graphics.FromImage(pic);
			string picName;

			pad.FillRectangle(whiteBrush, 0, 0, w, h);
			pad.DrawRectangle(grayPen, 0, 0, w - 1, h - 1);

			Vertex selectedVertex = curVertex;
			
			foreach(Edge e in Edges)
			{
				pad.DrawLine(dgPen, (int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
			}

			//
			// Draw text labels
			//
			foreach(DictionaryEntry de in Vertices)
			{
				Vertex v = (Vertex)de.Value;

				if (v != curVertex)
				{
					pad.DrawString(v.Label, new Font(FontFamily.GenericSansSerif, 7, FontStyle.Regular), 
						darkGreenBrush, new PointF((float)(v.XLap + 0.0), (float)(v.YLap + 0.0)));
				}
			}
			
			//
			// Draw connected component vertices
			//
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (v == selectedVertex) 
				{
					// Do nothing
				} 
				else if (Vertices.ContainsKey(v.ID)) 
				{
					pad.FillEllipse(siennaBrush, (int)v.XLap - 4, (int)v.YLap - 4, 8, 8);
				} 
				else 
				{
					throw new Exception("ConnectedComponent.drawLaplacianGraph: We shouldn't have other vertices to draw.");
				}
			}

			//
			// Draw connected edges
			//
			foreach(Edge e in selectedVertex.InEdges) 
			{
				pad.DrawLine(orangePen, (int)e.v1.XLap, (int)e.v1.YLap, (int)e.v2.XLap, (int)e.v2.YLap);
			}

			//
			// Draw connected vertices
			//
			foreach(DictionaryEntry de in Vertices) 
			{
				Vertex v = (Vertex)(de.Value);

				if (selectedVertex.ConnectedTo.Contains(v))
				{
					pad.FillEllipse(redBrush, (int)v.XLap - 4, (int)v.YLap - 4, 8, 8);
				}
			}
			

			//
			// Color selected vertex
			//
			PointF[] points = new PointF[3];
			points[0] = new PointF((float)curVertex.XLap, (float)curVertex.YLap - (float)(10.0f*Math.Sqrt(3)*(1.0f/3.0f)));
			points[1] = new PointF((float)curVertex.XLap - 7.0f, (float)curVertex.YLap + (float)(5.0f*Math.Sqrt(3)));
			points[2] = new PointF((float)curVertex.XLap + 7.0f, (float)curVertex.YLap + (float)(5.0f*Math.Sqrt(3)));
			pad.FillPolygon(redBrush, points);
			pad.DrawString(curVertex.Label, new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold), 
				darkGreenBrush, new PointF((float)(curVertex.XLap + 3.0), (float)(curVertex.YLap - 9.0)));

			Random r = new Random();
			picName = "graphLaplacian" + r.Next() + ".gif";
			pic.Save(graphDir + picName, ImageFormat.Gif);
			Graph.addFile(graphDir + picName);
			LaplacianGraphLocation = "graphs/" + picName;
		}
	}


	/// <summary>
	/// Represents the spectral information that a vertex holds in relation to its connected component.
	/// Every Vertex holds this class in .ComponentInfo
	/// </summary>
	public class EigenInformation 
	{
		/// <summary>
		/// The x-value of the vertex's graph coordinates for this eigenset (adj, lap, normlap).
		/// </summary>
		public double[] x;

		/// <summary>
		/// The y-value of the vertex's graph coordinates for this eigenset (adj, lap, normlap).
		/// </summary>
		public double[] y;

		/// <summary>
		/// Constructs an EigenInformation and allocates enough space for 4 graph (0-3rd eigenvector).
		/// </summary>
		/// <param name="n">Unused.</param>
		public EigenInformation(int n)
		{
			x = new double[4];
			y = new double[4];
		}
	}


	/// <summary>
	/// The SparseMatrix will be indexed FROM 0 TO SIZE-1, IT WILL **NOT** be indexed by node ID.  This is intended
	/// to be a GENERIC class.
	/// </summary>
	public class SparseMatrix 
	{
		/// <summary>
		/// The sortedlist that is this sparsematrix.
		/// </summary>
		SortedList[] adj;

		/// <summary>
		/// Gets M[i,j]
		/// </summary>
		/// <param name="i">First index.</param>
		/// <param name="j">Second index.</param>
		/// <returns>The value in the matrix.</returns>
		public int getValue(int i, int j) {
			if (adj[i].ContainsKey(j)) {
				return (int)(adj[i][j]);
			}

			return 0; // not found
		}

		/// <summary>
		/// Sets M[i,j]
		/// </summary>
		/// <param name="i">First index.</param>
		/// <param name="j">Second index.</param>
		/// <param name="value">The value in the matrix.</param>
		public void setValue(int i, int j, int value) {
			if (adj[i].ContainsKey(j)) {
				adj[i][j] = value;
			} else {
				adj[i].Add(j, value);
			}
		}

		/// <summary>
		/// Constructs a SparseMatrix.
		/// </summary>
		/// <param name="size">The size of this SparseMatrix.  *NOT* an estimate!</param>
		public SparseMatrix(int size) {
			adj = new SortedList[size];

			for(int i = 0; i < size; i++) {
				adj[i] = new SortedList();
			}
		}
	}
}